/********************************************************************/
/*                                                                  */
/*   MQPUT2 has 1 input parameter                                   */
/*                                                                  */
/*      -f name of the parameter input file                          */
/*                                                                  */
/*   The input parameter file contains all values, including the    */
/*   name of the queue and queue manager to write data to, the      */
/*   total number of messages to write, any MQMD parameters and     */
/*   a list of files which contain the message data.                */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/*                                                                  */
/* Changes in V1.01                                                 */
/*                                                                  */
/* 1) fixed bug with groupidx parameter overlaying qname.           */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/*                                                                  */
/* Changes in V1.1                                                  */
/*                                                                  */
/* 1) Added support for psc and jms folders in RFH2.                */
/* 2) Added support for Sun Solaris.                                */
/* 3) Use keyword rather than positional arguments.                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/*                                                                  */
/* Changes in V1.11                                                 */
/*                                                                  */
/* 1) fixed bug with argument overrides being ignored.              */
/*                                                                  */
/********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#ifdef WIN32
#include <windows.h>
#endif

#ifdef SOLARIS
#include <ctype.h>
#include <unistd.h>
#endif

#ifndef _WIN32
void Sleep(int amount)
{
	usleep(amount*1000);
}
#endif

/* includes for MQI */
#include <cmqc.h>

/* The following two defines are necessary to support MQSeries V5.1 and earlier */
#ifndef MQMT_MQE_FIELDS_FROM_MQE
#define MQMT_MQE_FIELDS_FROM_MQE 112L
#endif

#ifndef MQMT_MQE_FIELDS
#define MQMT_MQE_FIELDS 113L
#endif

/* default code page to use */
#ifdef WIN32
#define DEF_CODEPAGE	437
#else
#define DEF_CODEPAGE	850
#endif

#define MAX_SYNC_ALLOW		5000

/* maximum parameter lengths */
#define MAX_TOPIC_LEN		1024
#define MAX_SUBPOINT_LEN	1024
#define MAX_FILTER_LEN		1024
#define MAX_PUBTIME_LEN		32
#define MAX_JMS_DEST		256
#define MAX_JMS_REPLY		256
#define MAX_JMS_CORRELID	32
#define MAX_JMS_GROUPID		32
#define MAX_JMS_EXPIRE		32
#define MAX_JMS_DELMODE		16
#define MAX_JMS_SEQ			16
#define MAX_JMS_PRIORITY	16

/* input parameter keywords */
#define QNAME				"QNAME"
#define QMGR				"QMGR"
#define	REMOTEQM			"REMOTEQM"
#define MSGCOUNT			"MSGCOUNT"
#define BATCHSIZE			"BATCHSIZE"
#define QDEPTH				"QDEPTH"
#define QMAX				"QMAX"
#define TUNE				"TUNE"
#define SLEEPTIME			"SLEEPTIME"
#define PERSIST				"PERSIST"
#define CODEPAGE			"CODEPAGE"
#define ENCODING			"ENCODING"
#define EXPIRY				"EXPIRY"
#define MSGTYPE				"MSGTYPE"
#define CORRELID			"CORRELID"
#define CORRELIDX			"CORRELIDX"
#define GROUPID				"GROUPID"
#define GROUPIDX			"GROUPIDX"
#define FORMAT				"FORMAT"
#define REPLYQM				"REPLYQM"
#define REPLYQ				"REPLYQ"
#define PRIORITY			"PRIORITY"
#define REPORT				"REPORT"
#define DELIMITER			"DELIMITER"
#define DELIMITERX			"DELIMITERX"
#define THINKTIME			"THINKTIME"
#define RFH					"RFH"
#define RFH_DOMAIN			"RFH_DOMAIN"
#define RFH_MSG_SET			"RFH_MSG_SET"
#define RFH_MSG_TYPE		"RFH_MSG_TYPE"
#define	RFH_MSG_FMT			"RFH_MSG_FMT"
#define	RFH_APP_GROUP		"RFH_APP_GROUP"
#define RFH_FORMAT			"RFH_FORMAT"
#define RFH_CCSID			"RFH_CCSID"
#define	RFH_ENCODING		"RFH_ENCODING"
#define RFH_NAME_CCSID		"RFH_NAME_CCSID"
#define RFH_PSC_REQTYPE		"RFH_PSC_REQTYPE"
#define RFH_PSC_TOPIC1		"RFH_PSC_TOPIC1"
#define RFH_PSC_TOPIC2		"RFH_PSC_TOPIC2"
#define RFH_PSC_TOPIC3		"RFH_PSC_TOPIC3"
#define RFH_PSC_TOPIC4		"RFH_PSC_TOPIC4"
#define RFH_PSC_SUBPOINT	"RFH_PSC_SUBPOINT"
#define RFH_PSC_FILTER		"RFH_PSC_FILTER"
#define RFH_PSC_REPLYQM		"RFH_PSC_REPLYQM"
#define RFH_PSC_REPLYQ		"RFH_PSC_REPLYQ"
#define RFH_PSC_PUBTIME		"RFH_PSC_PUBTIME"
#define RFH_PSC_SEQNO		"RFH_PSC_SEQNO"
#define RFH_PSC_LOCAL		"RFH_PSC_LOCAL"
#define RFH_PSC_NEWONLY		"RFH_PSC_NEWONLY"
#define RFH_PSC_OTHERONLY	"RFH_PSC_OTHERONLY"
#define RFH_PSC_ONDEMAND	"RFH_PSC_ONDEMAND"
#define RFH_PSC_RETAINPUB	"RFH_PSC_RETAINPUB"
#define RFH_PSC_ISRETAINPUB	"RFH_PSC_ISRETAINPUB"
#define RFH_PSC_CORRELID	"RFH_PSC_CORRELID"
#define RFH_PSC_DEREGALL	"RFH_PSC_DEREGALL"
#define RFH_PSC_INFRETAIN	"RFH_PSC_INFRETAIN"
#define RFH_JMS_REQTYPE		"RFH_JMS_REQTYPE"
#define RFH_JMS_DEST		"RFH_JMS_DEST"
#define RFH_JMS_REPLY		"RFH_JMS_REPLY"
#define RFH_JMS_CORRELID	"RFH_JMS_CORRELID"
#define RFH_JMS_GROUPID		"RFH_JMS_GROUPID"
#define RFH_JMS_PRIORITY	"RFH_JMS_PRIORITY"
#define RFH_JMS_EXPIRE		"RFH_JMS_EXPIRE"
#define RFH_JMS_DELMODE		"RFH_JMS_DELMODE"
#define RFH_JMS_SEQ			"RFH_JMS_SEQ"
#define HEADER		"[HEADER]"
#define FILELIST	"[FILELIST]"

#define INIT_COUNT	32		/* number of messages written initially */
#define MIN_SLEEP	10
#define MAX_SLEEP	10000
#define MAX_RFH_DATA_LEN	64

#define	RFH_NO		0
#define RFH_V1		1
#define RFH_V2		2
#define RFH_XML		3
#define RFH_AUTO	4

#define RFH_PSC_SUB			1
#define RFH_PSC_UNSUB		2
#define RFH_PSC_PUB			3
#define RFH_PSC_REQPUB		4
#define RFH_PSC_DELPUB		5

#define RFH_JMS_TEXT		1
#define RFH_JMS_BYTES		2
#define RFH_JMS_STREAM		3
#define RFH_JMS_OBJECT		4
#define RFH_JMS_MAP			5
#define RFH_JMS_NONE		6

#define RFH2_MCD_BEGIN "<mcd>"
#define RFH2_MCD_END "</mcd>"
#define RFH2_MSD_BEGIN "<Msd>"
#define RFH2_MSD_END "</Msd>"
#define RFH2_SET_BEGIN "<Set>"
#define RFH2_SET_END "</Set>"
#define RFH2_TYPE_BEGIN "<Type>"
#define RFH2_TYPE_END "</Type>"
#define RFH2_FMT_BEGIN "<Fmt>"
#define RFH2_FMT_END "</Fmt>"

#define RFH2_PSC_BEGIN "<psc>"
#define RFH2_PSC_END "</psc>"
#define RFH2_PSC_COMMAND_BEGIN	"<Command>"
#define RFH2_PSC_COMMAND_END	"</Command>"
#define RFH2_PSC_TOPIC_BEGIN	"<Topic>"
#define RFH2_PSC_TOPIC_END		"</Topic>"
#define RFH2_PSC_SUBPOINT_BEGIN	"<SubPoint>"
#define RFH2_PSC_SUBPOINT_END	"</SubPoint>"
#define RFH2_PSC_FILTER_BEGIN	"<Filter>"
#define RFH2_PSC_FILTER_END	"</Filter>"
#define RFH2_PSC_QMGRNAME_BEGIN	"<QMgrName>"
#define RFH2_PSC_QMGRNAME_END	"</QMgrName>"
#define RFH2_PSC_QNAME_BEGIN	"<QName>"
#define RFH2_PSC_QNAME_END		"</QName>"
#define RFH2_PSC_PUBTIME_BEGIN	"<PubTime>"
#define RFH2_PSC_PUBTIME_END	"</PubTime>"
#define RFH2_PSC_SEQNUM_BEGIN	"<SeqNum>"
#define RFH2_PSC_SEQNUM_END		"</SeqNum>"
#define RFH2_PSC_REGOPT_BEGIN	"<RegOpt>"
#define RFH2_PSC_REGOPT_END		"</RegOpt>"
#define RFH2_PSC_PUBOPT_BEGIN	"<PubOpt>"
#define RFH2_PSC_PUBOPT_END		"</PubOpt>"
#define RFH2_PSC_DELOPT_BEGIN	"<DelOpt>"
#define RFH2_PSC_DELOPT_END		"</DelOpt>"
#define RFH2_PSC_LOCAL			"Local"
#define RFH2_PSC_NEWONLY		"NewPubsOnly"
#define RFH2_PSC_OTHERONLY		"OtherSubsOnly"
#define RFH2_PSC_ONDEMAND		"PubOnReqOnly"
#define RFH2_PSC_RETAINPUB		"RetainPub"
#define RFH2_PSC_ISRETAINPUB	"IsRetainedPub"
#define RFH2_PSC_CORRELASID		"CorrelAsId"
#define RFH2_PSC_DEREGALL		"DeregAll"
#define RFH2_PSC_INFORMIFRET	"InformIfRet"
#define RFH2_PSC_PERS			"Pers"
#define RFH2_PSC_NON_PERS		"NonPers"
#define RFH2_PSC_PERSASPUB		"PersAsPub"
#define RFH2_PSC_PERSASQUEUE	"PersAsQueue"

#define RFH_PSC_PERS			1
#define RFH_PSC_NON_PERS		2
#define RFH_PSC_PERS_PUB		3
#define RFH_PSC_PERS_QUEUE		4

#define PSC_REGSUB		"RegSub"
#define PSC_DEREGSUB	"DeregSub"
#define PSC_PUBLISH		"Publish"
#define PSC_REQUPDATE	"ReqUpdate"
#define PSC_DELETEPUB	"DeletePub"

#define RFH2_JMS_BEGIN "<jms>"
#define RFH2_JMS_END "</jms>"
#define RFH2_JMS_DST_BEGIN "<Dst>"
#define RFH2_JMS_DST_END "</Dst>"
#define RFH2_JMS_RTO_BEGIN "<Rto>"
#define RFH2_JMS_RTO_END "</Rto>"
#define RFH2_JMS_PRI_BEGIN "<Pri>"
#define RFH2_JMS_PRI_END "</Pri>"
#define RFH2_JMS_DLV_BEGIN "<Dlv>"
#define RFH2_JMS_DLV_END "</Dlv>"
#define RFH2_JMS_EXP_BEGIN "<Exp>"
#define RFH2_JMS_EXP_END "</Exp>"
#define RFH2_JMS_CID_BEGIN "<Cid>"
#define RFH2_JMS_CID_END "</Cid>"
#define RFH2_JMS_GID_BEGIN "<Gid>"
#define RFH2_JMS_GID_END "</Gid>"
#define RFH2_JMS_SEQ_BEGIN "<Seq>"
#define RFH2_JMS_SEQ_END "</Seq>"

#define JMS_TEXT		"jms_text"
#define JMS_BYTES		"jms_bytes"
#define JMS_OBJECT		"jms_object"
#define JMS_STREAM		"jms_stream"
#define JMS_MAP			"jms_map"
#define JMS_NONE		"jms_none"

#define CHAR_ASCII		0
#define CHAR_EBCDIC		1

typedef struct {
	void*	nextfile;
	char	*dataptr;
	int		length;
	int		hasRFH;
	int		useFileRFH;
	int		Codepage;
	int		Encoding;
	int		Expiry;
	int		Persist;
	int		Priority;
	int		Report;
	int		Msgtype;
	int		CorrelIdSet;
	int		GroupIdSet;
	int		FormatSet;
	char	*acqStorAddr;
	char	CorrelId[MQ_CORREL_ID_LENGTH + 1];
	char	GroupId[MQ_GROUP_ID_LENGTH + 1];
	char	Format[MQ_FORMAT_LENGTH + 1];
	char	ReplyQ[MQ_Q_NAME_LENGTH + 1];
	char	ReplyQM[MQ_Q_MGR_NAME_LENGTH + 1];
}	FILEPTR;

static char copyright[] = "(C) Copyright IBM Corp, 2001, 2002";
static char Version[]=\
"@(#)MQPut2 V1.11 - WMQI V2.1 Performance driver test tool  - Jim MacNair ";

#ifdef _DEBUG
static char Level[]="mqput2.c V1.11 Debug version ("__DATE__" "__TIME__")";
#else
static char Level[]="mqput2.c V1.11 Release version ("__DATE__" "__TIME__")";
#endif

	MQHCONN			qm=0;			/* queue manager connection handle */
	MQHOBJ			q=0;			/* queue handle used for mqputs    */
	MQHOBJ			Hinq;			/* inquire object handle           */
	static int		filelist = 0;	/* found [filelist] separator      */
	int				fileCount=0;	/* number of files read            */
	int				mesgCount=0;	/* number of messages found        */

/* global error switch */
	int		err=0;

/**************************************************************/
/*                                                            */
/* Parameter values from parameter file.                      */
/*                                                            */
/**************************************************************/

	/* Universal parameters */
	int				persist = 0;
	int				priority = MQPRI_PRIORITY_AS_Q_DEF;
	int				batchsize = 0;	/* no units of work */
	int				saveBatchSize=0;
	int				qdepth = 0;
	int				qmax = 0;
	int				sleeptime = 10;
	int				totcount=1;
	int				saveTotcount=0;
	int				tune=0;
	int				msgwritten=0;

	/* MQMD parameters - per message */
	int				codepage = 437;
	int				encoding = 546;
	int				expiry=MQEI_UNLIMITED;
	int				msgtype=0;
	int				report=0;
	int				correlidSet=0;
	int				groupidSet=0;
	int				formatSet=0;

	/* RFH parameters - per message */
	int				rfh=RFH_AUTO;
	int				rfhlength=0;
	int				rfhccsid=437;
	int				rfhencoding=546;
	int				rfh_psc_seqno=0;
	char			rfh_psc_reqtype=0;
	char			rfh_jms_reqtype=0;
	char			rfh_psc_local=0;
	char			rfh_psc_newonly=0;
	char			rfh_psc_otheronly=0;
	char			rfh_psc_ondemand=0;
	char			rfh_psc_retainpub=0;
	char			rfh_psc_isretainpub=0;
	char			rfh_psc_correlid=0;
	char			rfh_psc_deregall=0;
	char			rfh_psc_infretain=0;
	char			rfh_psc_pers_type=0;
#ifdef SOLARIS
	unsigned char	rfhdata[8192];
#else
	char			rfhdata[8192];
#endif
	char			rfhdomain[MAX_RFH_DATA_LEN + 1];
	char			rfhset[MAX_RFH_DATA_LEN + 1];
	char			rfhtype[MAX_RFH_DATA_LEN + 1];
	char			rfhfmt[MAX_RFH_DATA_LEN + 1];
	char			rfhappgroup[MAX_RFH_DATA_LEN + 1];
	char			rfhformat[MAX_RFH_DATA_LEN + 1];
	char			rfh_PscReplyQ[MQ_Q_NAME_LENGTH + 1];
	char			rfh_PscReplyQM[MQ_Q_MGR_NAME_LENGTH + 1];
	char			rfh_topic1[MAX_TOPIC_LEN];
	char			rfh_topic2[MAX_TOPIC_LEN];
	char			rfh_topic3[MAX_TOPIC_LEN];
	char			rfh_topic4[MAX_TOPIC_LEN];
	char			rfh_subpoint[MAX_SUBPOINT_LEN];
	char			rfh_filter[MAX_FILTER_LEN];
	char			rfh_pubtime[MAX_PUBTIME_LEN];
	char			rfh_jms_dest[MAX_JMS_DEST];
	char			rfh_jms_reply[MAX_JMS_REPLY];
	char			rfh_jms_correlid[MAX_JMS_CORRELID];
	char			rfh_jms_groupid[MAX_JMS_GROUPID];
	char			rfh_jms_expire[MAX_JMS_EXPIRE];
	char			rfh_jms_delmode[MAX_JMS_DELMODE];
	char			rfh_jms_seq[MAX_JMS_SEQ];
	char			rfh_jms_priority[MAX_JMS_PRIORITY];

	/* delimiter value, if specified */
	int				delimiterLen;
	char			delimiter[32];

	/* MQMD parameters - per message */
	char			correlid[MQ_CORREL_ID_LENGTH + 1];
	char			groupid[MQ_GROUP_ID_LENGTH + 1];
	char			msgformat[MQ_FORMAT_LENGTH + 1];
	char			replyQ[MQ_Q_NAME_LENGTH + 1];
	char			replyQM[MQ_Q_MGR_NAME_LENGTH + 1];

	/* Queue names to write to - universal */
	char			qname[MQ_Q_NAME_LENGTH + 1];
	char			qmname[MQ_Q_MGR_NAME_LENGTH + 1];
	char			saveQname[MQ_Q_NAME_LENGTH + 1];
	char			saveQMname[MQ_Q_MGR_NAME_LENGTH + 1];
	char			remoteQM[MQ_Q_MGR_NAME_LENGTH + 1];

	/* name of the parameters file */
	char	parmFilename[512];

static const unsigned char HEX_NUMBERS[] = "0123456789ABCDEF";

/*****************************************************************************/
static  const unsigned char aetab[] = {    /***** ASCII -----> EBCDIC *****/
                    0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,   /* 00 */
                    0x08,0x09,0x25,0x0B,0x0C,0x0D,0x0E,0x0F,   /* 08 */
                    0x10,0x11,0x12,0x13,0xB6,0xB5,0x16,0x17,   /* 10 */
                    0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,   /* 18 */
                    0x40,0x4F,0x7F,0x7B,0x5B,0x6C,0x50,0x7D,   /* 20 */
                    0x4D,0x5D,0x5C,0x4E,0x6B,0x60,0x4B,0x61,   /* 28 */
                    0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,   /* 30 */
                    0xF8,0xF9,0x7A,0x5E,0x4C,0x7E,0x6E,0x6F,   /* 38 */
                    0x7C,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,   /* 40 */
                    0xC8,0xC9,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,   /* 48 */
                    0xD7,0xD8,0xD9,0xE2,0xE3,0xE4,0xE5,0xE6,   /* 50 */
                    0xE7,0xE8,0xE9,0x4A,0xE0,0x5A,0x5F,0x6D,   /* 58 */
                    0x79,0x81,0x82,0x83,0x84,0x85,0x86,0x87,   /* 60 */
                    0x88,0x89,0x91,0x92,0x93,0x94,0x95,0x96,   /* 68 */
                    0x97,0x98,0x99,0xA2,0xA3,0xA4,0xA5,0xA6,   /* 70 */
                    0xA7,0xA8,0xA9,0xC0,0xBB,0xD0,0xA1,0x41,   /* 78 */
                    0x68,0xDC,0x51,0x42,0x43,0x44,0x47,0x48,   /* 80 */
                    0x52,0x53,0x54,0x57,0x56,0x58,0x63,0x67,   /* 88 */
                    0x71,0x9C,0x9E,0xCB,0xCC,0xCD,0xDB,0xDD,   /* 90 */
                    0xDF,0xEC,0xFC,0x70,0xB1,0x80,0x41,0xB4,   /* 98 */
                    0x45,0x55,0xCE,0xDE,0x49,0x69,0x9A,0x9B,   /* A0 */
                    0xAB,0xAF,0xBA,0xB8,0xB7,0xAA,0x8A,0x8B,   /* A8 */
                    0x41,0x41,0x41,0x41,0x41,0x65,0x62,0x64,   /* B0 */
                    0x41,0x41,0x41,0x41,0x41,0xB0,0xB2,0x41,   /* B8 */
                    0x41,0x41,0x41,0x41,0x41,0x41,0x46,0x66,   /* C0 */
                    0x41,0x41,0x41,0x41,0x41,0x41,0x41,0x9F,   /* C8 */
                    0x8C,0xAC,0x72,0x73,0x74,0xDA,0x75,0x76,   /* D0 */
                    0x77,0x41,0x41,0x41,0x41,0x6A,0x78,0x41,   /* D8 */
                    0xEE,0x59,0xEB,0xED,0xCF,0xEF,0xA0,0xAE,   /* E0 */
                    0x8E,0xFE,0xFB,0xFD,0x8D,0xAD,0xBC,0xBE,   /* E8 */
                    0xCA,0x8F,0xBF,0xB9,0xB6,0xB5,0x41,0x9D,   /* F0 */
                    0x90,0xBD,0xB3,0x41,0xFA,0xEA,0x41,0x41 }; /* F8 */

static  const unsigned char eatab[] = {  /***** EBCDIC -----> ASCII *****/
                    0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,   /* 00 */
                    0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,   /* 08 */
                    0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,   /* 10 */
                    0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,   /* 18 */
                    0x20,0x21,0x22,0x23,0x24,0x0A,0x26,0x27,   /* 20 */
                    0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,   /* 28 */
                    0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,   /* 30 */
                    0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,   /* 38 */
                    0x20,0xFF,0x83,0x84,0x85,0xA0,0xC6,0x86,   /* 40 */
                    0x87,0xA4,0x5B,0x2E,0x3C,0x28,0x2B,0x21,   /* 48 */
                    0x26,0x82,0x88,0x89,0x8A,0xA1,0x8C,0x8B,   /* 50 */
                    0x8D,0xE1,0x5D,0x24,0x2A,0x29,0x3B,0x5E,   /* 58 */
                    0x2D,0x2F,0xB6,0x8E,0xB7,0xB5,0xC7,0x8F,   /* 60 */
                    0x80,0xA5,0xDD,0x2C,0x25,0x5F,0x3E,0x3F,   /* 68 */
                    0x9B,0x90,0xD2,0xD3,0xD4,0xD6,0xD7,0xD8,   /* 70 */
                    0xDE,0x60,0x3A,0x23,0x40,0x27,0x3D,0x22,   /* 78 */
                    0x9D,0x61,0x62,0x63,0x64,0x65,0x66,0x67,   /* 80 */
                    0x68,0x69,0xAE,0xAF,0xD0,0xEC,0xE8,0xF1,   /* 88 */
                    0xF8,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,0x70,   /* 90 */
                    0x71,0x72,0xA6,0xA7,0x91,0xF7,0x92,0xCF,   /* 98 */
                    0xE6,0x7E,0x73,0x74,0x75,0x76,0x77,0x78,   /* A0 */
                    0x79,0x7A,0xAD,0xA8,0xD1,0xED,0xE7,0xA9,   /* A8 */
                    0xBD,0x9C,0xBE,0xFA,0x9F,0xF5,0xF4,0xAC,   /* B0 */
                    0xAB,0xF3,0xAA,0x7C,0xEE,0xF9,0xEF,0xF2,   /* B8 */
                    0x7B,0x41,0x42,0x43,0x44,0x45,0x46,0x47,   /* C0 */
                    0x48,0x49,0xF0,0x93,0x94,0x95,0xA2,0xE4,   /* C8 */
                    0x7D,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,0x50,   /* D0 */
                    0x51,0x52,0xD5,0x96,0x81,0x97,0xA3,0x98,   /* D8 */
                    0x5C,0xE1,0x53,0x54,0x55,0x56,0x57,0x58,   /* E0 */
                    0x59,0x5A,0xFD,0xE2,0x99,0xE3,0xE0,0xE5,   /* E8 */
                    0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,   /* F0 */
                    0x38,0x39,0xFC,0xEA,0x9A,0xEB,0xE9,0xFF};  /* F8 */

/**************************************************************/
/*                                                            */
/*Translate from ASCII to EBCDIC                              */
/*                                                            */
/**************************************************************/

void AsciiToEbcdic(unsigned char *dato, unsigned char *dati, unsigned int pl)

{
	unsigned int i;

	for(i=0; i<pl; i++)
	{
		dato[i] = aetab[dati[i]];
	}
}

/**************************************************************/
/*                                                            */
/*Translate from EBCDIC to ASCII                              */
/*                                                            */
/**************************************************************/

void EbcdicToAscii(unsigned char *dato, const unsigned char *dati, unsigned int pl)

{
	unsigned int i;
	for(i=0;i<pl;i++)
	{
		dato[i]=eatab[dati[i]];
	}
}

/**************************************************************/
/*                                                            */
/*Translate from ASCII to Hex                                 */
/*                                                            */
/**************************************************************/


void AsciiToHex(unsigned char *dato, unsigned char *dati, unsigned int pl)
{
	unsigned int i;
	unsigned int buffer;
	char	ch;

	buffer = 0;
	i = 0;
	while (i < (pl*2))
	{
		ch = (unsigned char) dati[buffer] >> 4;
		ch = HEX_NUMBERS[ch];
		dato[i++] = ch;

		ch = (unsigned char) dati[buffer] & 0x0F;
		ch = HEX_NUMBERS[ch];
		dato[i++] = ch;
		buffer++;
	}
}

/**************************************************************/
/*                                                            */
/* This routine skips blanks in a string                      */
/*                                                            */
/**************************************************************/

char * skipBlanks(char *str)

{
   while (*str == ' ')
   {
      str++;
   } /* endwhile */

   return str;
}

/**************************************************************/
/*                                                            */
/* This routine finds the next blank in a string              */
/*                                                            */
/**************************************************************/

char * findBlank(char *str)

{
   while ((str[0] != ' ') && (str[0] != 0))
   {
      str++;
   } /* endwhile */

   return str;
}

/**************************************************************/
/*                                                            */
/* This routine translates a string to upper case.            */
/*                                                            */
/**************************************************************/

void strupper(char *str)

{
	int	i=0;
	int	len=strlen(str);

	while (i < len)
	{
		str[i] = toupper(str[i]);
		i++;
	}
}

/**************************************************************/
/*                                                            */
/* Routine to replace certain special characters with their   */
/* respective escape sequences, as follows:                   */
/*                                                            */
/* & &amp;                                                    */
/* " &quot;                                                   */
/* < &gt;                                                     */
/* > &lt;                                                     */
/* ' &apos;                                                   */
/*                                                            */
/**************************************************************/

void replaceChars(const char *value, char *valStr)

{
	int		i=0;
	int		j=0;
	int		len;

	/* get the total length of the input string */
	len = strlen(value);

	while (i < len)
	{
		switch (value[i])
		{
		case '&':
			{
				valStr[j++] = '&';
				valStr[j++] = 'a';
				valStr[j++] = 'm';
				valStr[j++] = 'p';
				valStr[j++] = ';';
				break;
			}
		case '<':
			{
				valStr[j++] = '&';
				valStr[j++] = 'l';
				valStr[j++] = 't';
				valStr[j++] = ';';
				break;
			}
		case '>':
			{
				valStr[j++] = '&';
				valStr[j++] = 'g';
				valStr[j++] = 't';
				valStr[j++] = ';';
				break;
			}
		case '"':
			{
				valStr[j++] = '&';
				valStr[j++] = 'q';
				valStr[j++] = 'u';
				valStr[j++] = 'o';
				valStr[j++] = 't';
				valStr[j++] = ';';
				break;
			}
		case '\'':
			{
				valStr[j++] = '&';
				valStr[j++] = 'a';
				valStr[j++] = 'p';
				valStr[j++] = 'o';
				valStr[j++] = 's';
				valStr[j++] = ';';
				break;
			}
		default:
			{
				valStr[j++] = value[i];
			}
		}

		i++;
	}

	/* terminate the result string */
	valStr[j] = 0;
}

/**************************************************************/
/*                                                            */
/* Return the value of an individual                          */
/* hexadecimal character                                      */
/* the returned value should be                               */
/* between 0 and 15 inclusive.                                */
/*                                                            */
/**************************************************************/

char getHexCharValue(unsigned char charIn)

{
	char	ch;
	char	result=0;

	if ((charIn > '0') && (charIn <= '9'))
	{
		result = charIn & 15;
	}
	else
	{
		ch = toupper(charIn);
		if ((ch >= 'A') && (ch <= 'F'))
		{
			result = ch - 'A' + 10;
		}
	}

	return result;
}

/**************************************************************/
/*                                                            */
/* Remove the leading and trailing quotes if present.         */
/*                                                            */
/**************************************************************/

char * removeQuotes(char *dati)

{
	char	ch;

	/* check for a leading quote */
	if (('"' == dati[0]) || ((char)148) == dati[0])
	{
		ch = dati[0];

		/* found leading quote, skip it */
		dati++;

		/* make sure the last character is a matching quote */
		if (dati[strlen(dati) - 1] == ch)
		{
			dati[strlen(dati) - 1] = 0;
		}
		else
		{
			printf("***** missing trailing quote for value %s\n", dati);
		}
	}

	return dati;
}

/**************************************************************/
/*                                                            */
/* Translate from Hex to ASCII                                */
/*                                                            */
/**************************************************************/

void HexToAscii(char *dati, int pl, char *dato)

{
	int i;
	int buffer;
	char	ch;

	buffer = 0;
	i = 0;
	while (i < pl)
	{
		ch = getHexCharValue((unsigned char) dati[i++]) << 4;
		ch += getHexCharValue((unsigned char) dati[i++]);
		dato[buffer++] = ch;
	}
}

/**************************************************************/
/* Routine to reverse the bytes in an int variable            */
/**************************************************************/

int reverseBytes4(int var)

{
    return ((unsigned int) var >> 24) + (((unsigned int) var & 0x00ff0000) >> 8) +
           (((unsigned int) var & 0x0000ff00) << 8) + (((unsigned int) var & 255) <<24);
}

/**************************************************************/
/*                                                            */
/* Routine to determine if a code page is ASCII or EBCDIC     */
/*                                                            */
/**************************************************************/

int getCcsidType(const int ccsid)

{
	int	result= CHAR_ASCII;

	if ((ccsid == 037) || (ccsid == 500) ||
		(ccsid == 924) || (ccsid == 1140) ||
		(ccsid == 273) || (ccsid == 1141) ||
		(ccsid == 273) || (ccsid == 1142) ||
		(ccsid == 277) || (ccsid == 1143) ||
		(ccsid == 278) || (ccsid == 1144) ||
		(ccsid == 284) || (ccsid == 1145) ||
		(ccsid == 285) || (ccsid == 1146) ||
		(ccsid == 297) || (ccsid == 1147) ||
		(ccsid == 871) || (ccsid == 1148) ||
		(ccsid == 870) || (ccsid == 1149))
	{
		result = CHAR_EBCDIC;
	}

	return result;
}

/**************************************************************/
/*                                                            */
/* Routine to construct an RFH V1 header.                     */
/*                                                            */
/**************************************************************/

int buildRFH1()

{
	unsigned int	varlength;
	MQRFH	tempRFH;
	char	tempfield[1024];

	/* get a pointer to the area to build the non-fixed part of the RFH */
	memset(tempfield, 0, sizeof(tempfield));

	/* move in the identifier */
	strcpy(tempfield, MQNVS_APPL_TYPE);

	/* get the application group name */
	strcat(tempfield, rfhappgroup);
	strcat(tempfield, " ");

	/* next, copy in the message type name */
	strcat(tempfield, MQNVS_MSG_TYPE);

	/* get the message type into a string */
	strcat(tempfield, rfhformat);

	/* calculate the length of the variable part of the rfh */
	varlength = strlen(tempfield);

	/* Round it up to a multiple of 4 if necessary */
	if ((varlength % 4) > 0)
	{
		varlength += 4 - (varlength % 4);
	}

	while (strlen(tempfield) < varlength)
	{
		strcat(tempfield, " ");
	}

	/* check if we need to translate the variable rfh data to EBCDIC */
	if (getCcsidType(rfhccsid) == CHAR_ASCII)
	{
		/* ASCII, no need to translate */
		memcpy(rfhdata + MQRFH_STRUC_LENGTH_FIXED, tempfield, varlength);
	}
	else
	{
		/* translate to EBCDIC */
#ifdef SOLARIS
		AsciiToEbcdic(rfhdata + MQRFH_STRUC_LENGTH_FIXED, (unsigned char *)tempfield, varlength);
#else
		AsciiToEbcdic(rfhdata + MQRFH_STRUC_LENGTH_FIXED, tempfield, varlength);
#endif
	}

	/* initialize the header */
	tempRFH.Version = MQRFH_VERSION_1;
	tempRFH.StrucLength = MQRFH_STRUC_LENGTH_FIXED + varlength;
	tempRFH.Flags = MQRFH_NONE;

	/* set the encoding for the user data */
	if (encoding > 0)
	{
		tempRFH.Encoding = rfhencoding;
	}
	else
	{
		/* use a default if not set */
		tempRFH.Encoding = MQENC_NATIVE;
	}

	/* get the code page of the user data */
	if (codepage > 0)
	{
		tempRFH.CodedCharSetId = rfhccsid;
	}
	else
	{
		/* use a default if not set */
		tempRFH.CodedCharSetId = 437;
	}

	/* find out if the character set is EBCDIC */
	if (getCcsidType(codepage) == CHAR_ASCII)
	{
		/* ASCII - no need to translate */
		memcpy(tempRFH.StrucId, MQRFH_STRUC_ID, sizeof(tempRFH.StrucId));
		memcpy(tempRFH.Format, msgformat, MQ_FORMAT_LENGTH);
	}
	else
	{
		/* EBCDIC - must translate from ASCII */
#ifdef SOLARIS
		AsciiToEbcdic((unsigned char *)tempRFH.StrucId, (unsigned char *)MQRFH_STRUC_ID, sizeof(tempRFH.StrucId));
		AsciiToEbcdic((unsigned char *)tempRFH.Format, (unsigned char *)msgformat, MQ_FORMAT_LENGTH);
#else
		AsciiToEbcdic(tempRFH.StrucId, MQRFH_STRUC_ID, sizeof(tempRFH.StrucId));
		AsciiToEbcdic(tempRFH.Format, msgformat, MQ_FORMAT_LENGTH);
#endif
	}

	/* check if the encoding is host integers */
#ifdef WIN32
	if ((encoding & MQENC_INTEGER_REVERSED) == 0)
#else
	if ((encoding & MQENC_INTEGER_REVERSED) != 0)
#endif
	{
		/* we need to reverse the integers in all of the RFH header fields */
		tempRFH.Version        = reverseBytes4(tempRFH.Version);
		tempRFH.StrucLength    = reverseBytes4(tempRFH.StrucLength);
		tempRFH.Encoding       = reverseBytes4(tempRFH.Encoding);
		tempRFH.CodedCharSetId = reverseBytes4(tempRFH.CodedCharSetId);
		tempRFH.Flags          = reverseBytes4(tempRFH.Flags);
	}

	/* copy the fixed portion of the header to the data area */
	memcpy(rfhdata, (char *) &tempRFH, MQRFH_STRUC_LENGTH_FIXED);

	return varlength + MQRFH_STRUC_LENGTH_FIXED;
}

int buildJMS(char * varArea)

{
	int		varlen=0;
	int		worklen;
	int		extra;
	char	tempValue[256];
	char	tempfield[1024];

	tempfield[0] = 0;

	if (strlen(rfh_jms_dest) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_JMS_DST_BEGIN);

		/* get the message domain name and append it to the XML message */
		replaceChars(rfh_jms_dest, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_JMS_DST_END);
	}

	if (strlen(rfh_jms_reply) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_JMS_RTO_BEGIN);

		/* get the message domain name and append it to the XML message */
		replaceChars(rfh_jms_reply, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_JMS_RTO_END);
	}

	if (strlen(rfh_jms_correlid) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_JMS_CID_BEGIN);

		/* get the message domain name and append it to the XML message */
		replaceChars(rfh_jms_correlid, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_JMS_CID_END);
	}

	if (strlen(rfh_jms_groupid) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_JMS_GID_BEGIN);

		/* get the message domain name and append it to the XML message */
		replaceChars(rfh_jms_groupid, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_JMS_GID_END);
	}

	if (strlen(rfh_jms_expire) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_JMS_EXP_BEGIN);

		/* get the message domain name and append it to the XML message */
		replaceChars(rfh_jms_groupid, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_JMS_EXP_END);
	}

	if (strlen(rfh_jms_delmode) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_JMS_DLV_BEGIN);

		/* get the message domain name and append it to the XML message */
		replaceChars(rfh_jms_delmode, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_JMS_DLV_END);
	}

	if (strlen(rfh_jms_seq) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_JMS_SEQ_BEGIN);

		/* get the message domain name and append it to the XML message */
		replaceChars(rfh_jms_seq, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_JMS_SEQ_END);
	}

	if (strlen(rfh_jms_priority) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_JMS_PRI_BEGIN);

		/* get the message domain name and append it to the XML message */
		replaceChars(rfh_jms_priority, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_JMS_PRI_END);
	}

	/* check if we found any jms fields */
	if (tempfield[0] != 0)
	{
		/* add the XML tags and data */
		strcpy(varArea + 4, RFH2_JMS_BEGIN);
		strcat(varArea + 4, tempfield);
		strcat(varArea + 4, RFH2_JMS_END);

		varlen = strlen(varArea + 4);

		/* round to a multiple of four bytes */
		extra = varlen % 4;
		if (extra > 0)
		{
			extra = 4 - extra;
			varlen += extra;

			while (extra > 0)
			{
				strcat(varArea + 4, " ");
				extra--;
			}
		}

		worklen = varlen;

		/* set the binary length field at the front of the data */
#ifdef WIN32
		if ((encoding & MQENC_INTEGER_REVERSED) == 0)
#else
		if ((encoding & MQENC_INTEGER_REVERSED) != 0)
#endif
		{
			/* we need to reverse the integers in all of the RFH header fields */
			worklen = reverseBytes4(varlen);
		}

		memcpy(varArea, (const void *) &worklen, 4);
		varlen += 4;
	}

	return varlen;
}

void appendOption(char * tempfield, const char * optfield)

{
	switch (rfh_psc_reqtype)
	{
	case RFH_PSC_SUB:
		{
			strcat(tempfield, RFH2_PSC_REGOPT_BEGIN);
			break;
		}
	case RFH_PSC_UNSUB:
		{
			strcat(tempfield, RFH2_PSC_REGOPT_BEGIN);
			break;
		}
	case RFH_PSC_PUB:
		{
			strcat(tempfield, RFH2_PSC_PUBOPT_BEGIN);
			break;
		}
	case RFH_PSC_REQPUB:
		{
			strcat(tempfield, RFH2_PSC_REGOPT_BEGIN);
			break;
		}
	case RFH_PSC_DELPUB:
		{
			strcat(tempfield, RFH2_PSC_DELOPT_BEGIN);
			break;
		}
	}

	strcat(tempfield, optfield);

	switch (rfh_psc_reqtype)
	{
	case RFH_PSC_SUB:
		{
			strcat(tempfield, RFH2_PSC_REGOPT_END);
			break;
		}
	case RFH_PSC_UNSUB:
		{
			strcat(tempfield, RFH2_PSC_REGOPT_END);
			break;
		}
	case RFH_PSC_PUB:
		{
			strcat(tempfield, RFH2_PSC_PUBOPT_END);
			break;
		}
	case RFH_PSC_REQPUB:
		{
			strcat(tempfield, RFH2_PSC_REGOPT_END);
			break;
		}
	case RFH_PSC_DELPUB:
		{
			strcat(tempfield, RFH2_PSC_DELOPT_END);
			break;
		}
	}
}

int buildPSC(char * varArea)

{
	int		varlen=0;
	int		worklen;
	int		extra;
	char	tempValue[256];
	char	tempfield[1024];

	tempfield[0] = 0;

	if (rfh_psc_reqtype > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_COMMAND_BEGIN);

		/* get the command and append it to the XML message */
		switch (rfh_psc_reqtype)
		{
		case RFH_PSC_SUB:
			{
				strcat(tempfield, PSC_REGSUB);
				break;
			}
		case RFH_PSC_UNSUB:
			{
				strcat(tempfield, PSC_DEREGSUB);
				break;
			}
		case RFH_PSC_PUB:
			{
				strcat(tempfield, PSC_PUBLISH);
				break;
			}
		case RFH_PSC_REQPUB:
			{
				strcat(tempfield, PSC_REQUPDATE);
				break;
			}
		case RFH_PSC_DELPUB:
			{
				strcat(tempfield, PSC_DELETEPUB);
				break;
			}
		}

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_COMMAND_END);
	}

	if (strlen(rfh_topic1) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_TOPIC_BEGIN);

		/* get the topic and append it to the XML message */
		replaceChars(rfh_topic1, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_TOPIC_END);
	}

	if (strlen(rfh_topic2) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_TOPIC_BEGIN);

		/* get the topic and append it to the XML message */
		replaceChars(rfh_topic2, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_TOPIC_END);
	}

	if (strlen(rfh_topic3) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_TOPIC_BEGIN);

		/* get the topic and append it to the XML message */
		replaceChars(rfh_topic3, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_TOPIC_END);
	}

	if (strlen(rfh_topic4) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_TOPIC_BEGIN);

		/* get the topic and append it to the XML message */
		replaceChars(rfh_topic4, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_TOPIC_END);
	}

	if (strlen(rfh_subpoint) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_SUBPOINT_BEGIN);

		/* get the subscription point and append it to the XML message */
		replaceChars(rfh_subpoint, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_SUBPOINT_END);
	}

	if (strlen(rfh_filter) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_FILTER_BEGIN);

		/* get the filter and append it to the XML message */
		replaceChars(rfh_filter, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_FILTER_END);
	}

	if (strlen(rfh_PscReplyQM) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_QMGRNAME_BEGIN);

		/* get the reply queue manager name and append it to the XML message */
		replaceChars(rfh_PscReplyQM, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_QMGRNAME_END);
	}

	if (strlen(rfh_PscReplyQ) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_QNAME_BEGIN);

		/* get the reply queue name and append it to the XML message */
		replaceChars(rfh_PscReplyQ, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_QNAME_END);
	}

	if (strlen(rfh_pubtime) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_PUBTIME_BEGIN);

		/* get the pub time and append it to the XML message */
		replaceChars(rfh_pubtime, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_PUBTIME_END);
	}

	if (rfh_psc_seqno > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_PSC_SEQNUM_BEGIN);

		/* get the sequence number in character format and append it to the XML message */
		sprintf(tempValue, "%d", rfh_psc_seqno);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_PSC_SEQNUM_END);
	}

	if (rfh_psc_local > 0)
	{
		appendOption(tempfield, RFH2_PSC_LOCAL);
	}

	if (rfh_psc_newonly > 0)
	{
		appendOption(tempfield, RFH2_PSC_NEWONLY);
	}

	if (rfh_psc_otheronly > 0)
	{
		appendOption(tempfield, RFH2_PSC_OTHERONLY);
	}

	if (rfh_psc_ondemand > 0)
	{
		appendOption(tempfield, RFH2_PSC_ONDEMAND);
	}

	if (rfh_psc_retainpub > 0)
	{
		appendOption(tempfield, RFH2_PSC_RETAINPUB);
	}

	if (rfh_psc_isretainpub > 0)
	{
		appendOption(tempfield, RFH2_PSC_ISRETAINPUB);
	}

	if (rfh_psc_correlid > 0)
	{
		appendOption(tempfield, RFH2_PSC_CORRELASID);
	}

	if (rfh_psc_deregall > 0)
	{
		appendOption(tempfield, RFH2_PSC_DEREGALL);
	}

	if (rfh_psc_infretain > 0)
	{
		appendOption(tempfield, RFH2_PSC_INFORMIFRET);
	}

	if (rfh_psc_pers_type > 0)
	{
		switch(rfh_psc_pers_type)
		{
		case RFH_PSC_PERS:
			{
				appendOption(tempfield, RFH2_PSC_PERS);
				break;
			}
		case RFH_PSC_NON_PERS:
			{
				appendOption(tempfield, RFH2_PSC_NON_PERS);
				break;
			}
		case RFH_PSC_PERS_PUB:
			{
				appendOption(tempfield, RFH2_PSC_PERSASPUB);
				break;
			}
		case RFH_PSC_PERS_QUEUE:
			{
				appendOption(tempfield, RFH2_PSC_PERSASQUEUE);
				break;
			}
		}
	}

	/* check if we found any psc fields */
	if (tempfield[0] != 0)
	{
		/* add the XML tags and data */
		strcpy(varArea + 4, RFH2_PSC_BEGIN);
		strcat(varArea + 4, tempfield);
		strcat(varArea + 4, RFH2_PSC_END);

		varlen = strlen(varArea + 4);

		/* round to a multiple of four bytes */
		extra = varlen % 4;
		if (extra > 0)
		{
			extra = 4 - extra;
			varlen += extra;

			while (extra > 0)
			{
				strcat(varArea + 4, " ");
				extra--;
			}
		}

		worklen = varlen;

		/* set the binary length field at the front of the data */
#ifdef WIN32
		if ((encoding & MQENC_INTEGER_REVERSED) == 0)
#else
		if ((encoding & MQENC_INTEGER_REVERSED) != 0)
#endif
		{
			/* we need to reverse the integers in all of the RFH header fields */
			worklen = reverseBytes4(varlen);
		}

		memcpy(varArea, (const void *) &worklen, 4);
		varlen += 4;
	}

	return varlen;
}

int buildMCD(char * varArea)

{
	int		varlen=0;
	int		worklen;
	int		extra;
	char	tempValue[256];
	char	tempfield[1024];

	tempfield[0] = 0;

	if (rfh_jms_reqtype > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_MSD_BEGIN);

		switch (rfh_jms_reqtype)
		{
		case RFH_JMS_TEXT:
			{
				strcat(tempfield, JMS_TEXT);
				break;
			}
		case RFH_JMS_BYTES:
			{
				strcat(tempfield, JMS_BYTES);
				break;
			}
		case RFH_JMS_OBJECT:
			{
				strcat(tempfield, JMS_OBJECT);
				break;
			}
		case RFH_JMS_STREAM:
			{
				strcat(tempfield, JMS_STREAM);
				break;
			}
		case RFH_JMS_MAP:
			{
				strcat(tempfield, JMS_MAP);
				break;
			}
		case RFH_JMS_NONE:
			{
				strcat(tempfield, JMS_NONE);
				break;
			}
		}

		/* Add the closing tag */
		strcat(tempfield, RFH2_MSD_END);
	}
	else
	{
		if (strlen(rfhdomain) > 0)
		{
			/* add the XML tag */
			strcat(tempfield, RFH2_MSD_BEGIN);

			/* get the message domain name and append it to the XML message */
			replaceChars(rfhdomain, tempValue);
			strcat(tempfield, tempValue);

			/* Add the closing tag */
			strcat(tempfield, RFH2_MSD_END);
		}
	}

	if (strlen(rfhset) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_SET_BEGIN);

		/* get the message set name and append it to the XML message */
		replaceChars(rfhset, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_SET_END);
	}

	if (strlen(rfhtype) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_TYPE_BEGIN);

		/* get the message type name and append it to the XML message */
		replaceChars(rfhtype, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_TYPE_END);
	}

	if (strlen(rfhfmt) > 0)
	{
		/* add the XML tag */
		strcat(tempfield, RFH2_FMT_BEGIN);

		/* get the message fmt name and append it to the XML message */
		replaceChars(rfhfmt, tempValue);
		strcat(tempfield, tempValue);

		/* Add the closing tag */
		strcat(tempfield, RFH2_FMT_END);
	}

	/* check if we found any mcd fields */
	if (tempfield[0] != 0)
	{
		/* add the XML tags and data */
		strcpy(varArea + 4, RFH2_MCD_BEGIN);
		strcat(varArea + 4, tempfield);
		strcat(varArea + 4, RFH2_MCD_END);

		varlen = strlen(varArea + 4);

		/* round to a multiple of four bytes */
		extra = varlen % 4;
		if (extra > 0)
		{
			extra = 4 - extra;
			varlen += extra;

			while (extra > 0)
			{
				strcat(varArea + 4, " ");
				extra--;
			}
		}

		worklen = varlen;

		/* set the binary length field at the front of the data */
#ifdef WIN32
		if ((encoding & MQENC_INTEGER_REVERSED) == 0)
#else
		if ((encoding & MQENC_INTEGER_REVERSED) != 0)
#endif
		{
			/* we need to reverse the integers in all of the RFH header fields */
			worklen = reverseBytes4(varlen);
		}

		memcpy(varArea, (const void *) &worklen, 4);
		varlen += 4;
	}

	return varlen;
}

/**************************************************************/
/*                                                            */
/* Routine to construct an RFH V2 header.                     */
/*                                                            */
/**************************************************************/

int buildRFH2()

{
	unsigned int	varlength=0;
	MQRFH2	tempRFH;
	char	tempfield[8192];

	/* get a pointer to the area to build the variable part of the RFH */
	memset(tempfield, 0, sizeof(tempfield));

	/* initialize the RFH length */
	rfhlength = MQRFH_STRUC_LENGTH_FIXED_2;

	/* check if we have a variable part */
	if (RFH_V2 == rfh)
	{
		tempfield[0] = 0;

		/* build any folders that are required */
		varlength += buildMCD(tempfield + varlength);
		varlength += buildPSC(tempfield + varlength);
		varlength += buildJMS(tempfield + varlength);

		/* Allow for the variable part length and data */
		rfhlength += varlength;
	}

	/* initialize the header */
	tempRFH.Version = MQRFH_VERSION_2;
	tempRFH.Flags = MQRFH_NONE;
	tempRFH.NameValueCCSID = 1208;
	tempRFH.StrucLength = MQRFH_STRUC_LENGTH_FIXED_2 + varlength;

	/* set the encoding for the user data */
	if (encoding > 0)
	{
		tempRFH.Encoding = rfhencoding;
	}
	else
	{
		tempRFH.Encoding = MQENC_NATIVE;
	}

	/* get the code page of the user data */
	if (codepage > 0)
	{
		tempRFH.CodedCharSetId = rfhccsid;
	}
	else
	{
		tempRFH.CodedCharSetId = DEF_CODEPAGE;
	}

	/* find out if the character set is EBCDIC */
	if (getCcsidType(codepage) == CHAR_ASCII)
	{
		memcpy(tempRFH.StrucId, MQRFH_STRUC_ID, sizeof(tempRFH.StrucId));
		memcpy(tempRFH.Format, msgformat, MQ_FORMAT_LENGTH);
	}
	else
	{
#ifdef SOLARIS
		AsciiToEbcdic((unsigned char *)tempRFH.StrucId, (unsigned char *)MQRFH_STRUC_ID, sizeof(tempRFH.StrucId));
		AsciiToEbcdic((unsigned char *)tempRFH.Format, (unsigned char *)msgformat, MQ_FORMAT_LENGTH);
#else
		AsciiToEbcdic(tempRFH.StrucId, MQRFH_STRUC_ID, sizeof(tempRFH.StrucId));
		AsciiToEbcdic(tempRFH.Format, msgformat, MQ_FORMAT_LENGTH);
#endif
	}

#ifdef WIN32
	if ((encoding & MQENC_INTEGER_REVERSED) == 0)
#else
	if ((encoding & MQENC_INTEGER_REVERSED) != 0)
#endif
	{
		/* we need to reverse the integers in all of the RFH header fields */
		tempRFH.Version        = reverseBytes4(tempRFH.Version);
		tempRFH.StrucLength    = reverseBytes4(tempRFH.StrucLength);
		tempRFH.Encoding       = reverseBytes4(tempRFH.Encoding);
		tempRFH.CodedCharSetId = reverseBytes4(tempRFH.CodedCharSetId);
		tempRFH.Flags          = reverseBytes4(tempRFH.Flags);
		tempRFH.NameValueCCSID = reverseBytes4(tempRFH.NameValueCCSID);
	}

	/* copy the fixed portion of the header and the length of the */
	/* variable part to the data area */
	memcpy(rfhdata, (char *) &tempRFH, MQRFH_STRUC_LENGTH_FIXED_2);

	/* check if we have a variable part */
	if (RFH_V2 == rfh)
	{
		memcpy(rfhdata + MQRFH_STRUC_LENGTH_FIXED_2, tempfield, varlength);
	}

	return rfhlength;
}

/**************************************************************/
/*                                                            */
/* Routine to create an RFH header, if necessary, to insert   */
/*  before the message data.  This routine will create an     */
/*  rfh if the rfh parameter is set to V1, V2 or XML.         */
/*                                                            */
/**************************************************************/

void createRFH()

{
	/* pad with blanks to make 8 characters */
	while (strlen(msgformat) < MQ_FORMAT_LENGTH)
	{
		strcat(msgformat, " ");
	}

	/* check if we need to build an RFH header to */
	/* insert before the message data for each message */
	switch (rfh)
	{
	case RFH_V1:
		{
			rfhlength = buildRFH1();
			break;
		}
	case RFH_V2:
		{
			rfhlength = buildRFH2();
			break;
		}
	case RFH_XML:
		{
			strcpy(msgformat, "xml     ");
			rfhlength = buildRFH2();
			break;
		}
	}
}

/**************************************************************/
/*                                                            */
/* Routine to check for an rfh in the file data.  This        */
/*  routine checks the first four characters for an RFH       */
/*  structure id in either ASCII or EBCDIC, and then checks   */
/*  the next four bytes for either a binary 1 or 2, trying    */
/*  both normal and reversed integers.                        */
/*                                                            */
/**************************************************************/

int findRFH(FILEPTR* fptr)

{
	int		rfhtype=0;
	int		rfhversion=0;
	MQRFH	*tempRFH;
	char	tempid[5];

	/* tell what we are doing */
	printf("Looking for rfh in file\n");

	/* is the message long enough to have an RFH? */
	if (fptr->length > MQRFH_STRUC_LENGTH_FIXED_2)
	{
		/* translate the first 4 bytes from EBCDIC to ASCII */
		EbcdicToAscii((unsigned char *) &tempid, (unsigned char *) fptr->dataptr, sizeof(MQRFH_STRUC_ID) - 1);

		/* check for either an ASCII or EBCDIC structure id */
		if ((memcmp(tempid, MQRFH_STRUC_ID, sizeof(MQRFH_STRUC_ID) - 1) == 0) ||
			(memcmp(fptr->dataptr, MQRFH_STRUC_ID, sizeof(MQRFH_STRUC_ID) - 1) == 0))
		{
			/* the first four bytes match an RFH structure id */
			tempRFH = (MQRFH *) fptr->dataptr;
			rfhversion = tempRFH->Version;
			
			/* check for the RFH version field in PC format */
			if ((MQRFH_VERSION_1 == rfhversion) || (MQRFH_VERSION_2 == rfhversion))
			{
				/* the version field is ok - asssume we have an rfh in the file */
				rfhtype = rfhversion;

				/* was encoding for rfh specified in parameters file? */
				if (encoding > 0)
				{
					/* yes, use it */
					fptr->Encoding = encoding;
				}
				else
				{
					/* pick an intelligent default - PC Intel = 546 */
					fptr->Encoding = MQENC_NATIVE;
				}
			}
			else
			{
				/* reverse the order, and check for host integers */
				rfhversion = reverseBytes4(rfhversion);

				/* check for the RFH version field in host format */
				if ((MQRFH_VERSION_1 == rfhversion) || (MQRFH_VERSION_2 == rfhversion))
				{
					rfhtype = rfhversion;

					/* was encoding for rfh specified in parameters file? */
					if (encoding > 0)
					{
						/* yes, use it */
						fptr->Encoding = encoding;
					}
					else
					{
#ifdef WIN32
						/* pick an intelligent default - PC = 546 */
						fptr->Encoding = MQENC_INTEGER_REVERSED + MQENC_DECIMAL_REVERSED + MQENC_FLOAT_IEEE_REVERSED;
#else
						/* pick an intelligent default - host S/390 = 785 */
						fptr->Encoding = MQENC_INTEGER_NORMAL + MQENC_DECIMAL_NORMAL + MQENC_FLOAT_S390;
#endif
					}
				}
			}

			/* did we find an rfh at the front of the file? */
			if (rfhtype > 0)
			{
				/* found an rfh header - set the code page if necessary */
				if (rfhccsid > 0)
				{
					/* use the code page specified in the parameters file */
					fptr->Codepage = codepage;
				}
				else
				{
					/* pick an intelligent default */
					if (memcmp(tempid, MQRFH_STRUC_ID, sizeof(MQRFH_STRUC_ID)) == 0)
					{
						/* set an EBCDIC default value */
						fptr->Codepage = 500;
					}
					else
					{
						/* set an ASCII default value */
						fptr->Codepage = DEF_CODEPAGE;
					}
				}
			}
		}
	}

	/* did we find an rfh at the front of the file? */
	if (rfhtype > 0)
	{
		/* found rfh */
		printf("using rfh version %d found in file\n", rfhtype);
	}
	else
	{
		/* no rfh found in file - rfh set to no */
		printf("no rfh found in file\n");
	}

	return rfhtype;
}

char * searchForDelimiter(char * msgPtr, const int maxlen)

{
	int		i=0;
	char *	newPtr=NULL;

	/* return if there is no delimiter specified */
	if (0 == delimiterLen)
	{
		return NULL;
	}

	while (1)
	{
		while ((i < (maxlen - delimiterLen)) && (msgPtr[i] != delimiter[0]))
		{
			i++;
		}

		/* did we run out of message to check? */
		if (i < (maxlen - delimiterLen))
		{
			/* we found a delimiter that we need to check */
			/* check if we have more delimiter characters to compare */
			if (1 == delimiterLen)
			{
				/* no, found a delimiter */
				/* set the new address pointer */
				newPtr = msgPtr + i;
				break;
			}
			else
			{
				/* do a full memory compare */
				if (0 == memcmp(msgPtr + i, delimiter, delimiterLen))
				{
					/* we have found a delimiter */
					newPtr = msgPtr + i;
					break;
				}
			}
		}
		else
		{
			break;
		}

		/* go on to the next entry */
		i++;
	}

	return newPtr;
}

/**************************************************************/
/*                                                            */
/* Read message data from a file.                             */
/*                                                            */
/**************************************************************/

int readFileData(char *filename, int *length, char ** dataptr)

{
	int	rc=0;
	int datalen;
	char *msgdata;
	char *ptr;
	FILE*	datafile;

	if ((datafile = fopen(filename, "rb")) == NULL)
	{
		printf("Unable to open input file %s\n", filename);
		exit(1);
	}

	/* figure out how long the data is and read it into a buffer */
	fseek(datafile, 0L, SEEK_END);
	datalen = ftell(datafile);
	fseek(datafile, 0L, SEEK_SET);
	msgdata = (char *) malloc(datalen + rfhlength + 1);
	ptr = msgdata;
	memset(msgdata, 0, datalen + rfhlength + 1);

	/* check if we need to add an RFH to the beginning of the data */
	if ((RFH_V1 == rfh) || (RFH_V2 == rfh) || (RFH_XML == rfh))
	{
		/* move the rfh header to the front of the data */
		memcpy(msgdata, rfhdata, rfhlength);

		/* read in the data after the rfh */
		ptr += rfhlength;
	}

	fread(ptr, 1, datalen, datafile);
	fclose(datafile);

	/* tell what we are doing */
	printf("\n%d bytes read from data file %s\n", datalen, filename);

	/* set the length and data pointers */
	(*length) = datalen + rfhlength;
	(*dataptr) = msgdata;

	return rc;
}

/**************************************************************/
/*                                                            */
/* Routine to scan for a delimiter sequence in the data.      */
/*                                                            */
/**************************************************************/

char * scanForDelim(char * msgdata, const int datalen)

{
	char *		ptr=NULL;
	char *		ptrend;

	/* do we have a delimiter? */
	if (delimiterLen > 0)
	{
		ptr = msgdata;
		ptrend = msgdata + datalen - delimiterLen;
		while ((ptr < ptrend) && (ptr != NULL) && (memcmp(ptr, delimiter, delimiterLen) != 0))
		{
			while ((ptr < ptrend) && (ptr[0] != delimiter[0]))
			{
				ptr++;
			}

			/* could not find first character of delimiter - time to get out */
			if (ptr[0] != delimiter[0])
			{
				ptr = NULL;
			}
			else
			{
				if ((ptr != NULL) && (memcmp(ptr, delimiter, delimiterLen) != 0))
				{
					ptr++;
				}
			}
		}
	}

	return ptr;
}

/**************************************************************/
/*                                                            */
/* Insert a message data file block into the file chain.      */
/*                                                            */
/**************************************************************/

FILEPTR * createFileBlock(char *msgPtr, char * allocPtr, const int msgLen)

{
	FILEPTR*	newfptr;
	char		tempid[2 * MQ_CORREL_ID_LENGTH + 1];
	char		tempid2[2 * MQ_GROUP_ID_LENGTH + 1];

	/* increase the message count */
	mesgCount++;

	/* allocate a new message data block */
	newfptr  = (FILEPTR *) malloc(sizeof(FILEPTR));
	newfptr->dataptr = msgPtr;
	newfptr->length = msgLen;
	newfptr->hasRFH = rfh;
	newfptr->useFileRFH = 0;
	newfptr->nextfile = NULL;
	newfptr->acqStorAddr = allocPtr;

	if ((RFH_V1 == rfh) || (RFH_V2 == rfh) || (RFH_XML == rfh))
	{
		newfptr->Codepage = rfhccsid;
		newfptr->Encoding = rfhencoding;
	}
	else
	{
		newfptr->Codepage = codepage;
		newfptr->Encoding = encoding;
	}

	newfptr->Expiry   = expiry;
	newfptr->Msgtype  = msgtype;
	newfptr->Persist  = persist;
	newfptr->Priority = priority;
	newfptr->Report = report;
	memset(newfptr->CorrelId, 0, sizeof(newfptr->CorrelId));
	newfptr->CorrelIdSet = correlidSet;
	if (correlidSet)
	{
		memcpy(newfptr->CorrelId, correlid, sizeof(newfptr->CorrelId));
	}

	memset(newfptr->GroupId, 0, sizeof(newfptr->GroupId));
	newfptr->GroupIdSet = groupidSet;
	if (groupidSet)
	{
		memcpy(newfptr->GroupId, groupid, sizeof(newfptr->GroupId));
	}

	memset(newfptr->Format, 0, sizeof(newfptr->Format));
	newfptr->FormatSet = formatSet;
	if (formatSet)
	{
		memcpy(newfptr->Format, msgformat, sizeof(newfptr->Format));
	}

	strcpy(newfptr->ReplyQM, replyQM);
	strcpy(newfptr->ReplyQ, replyQ);

	if (RFH_AUTO == rfh)
	{
		newfptr->hasRFH = findRFH(newfptr);

		if (RFH_NO != newfptr->hasRFH)
		{
			newfptr->useFileRFH = 1;
		}
	}

	if (1 == correlidSet)
	{
		memset(tempid, 0, sizeof(tempid));
#ifdef SOLARIS
		AsciiToHex((unsigned char *)tempid, (unsigned char *)correlid, MQ_CORREL_ID_LENGTH);
#else
		AsciiToHex(tempid, correlid, MQ_CORREL_ID_LENGTH);
#endif
		printf("Correlation id set to %s\n", tempid);
	}

	if (1 == groupidSet)
	{
		memset(tempid2, 0, sizeof(tempid2));
#ifdef SOLARIS
		AsciiToHex((unsigned char *)tempid2, (unsigned char *)groupid, MQ_GROUP_ID_LENGTH);
#else
		AsciiToHex(tempid2, groupid, MQ_GROUP_ID_LENGTH);
#endif
		printf("Group id set to %s\n", tempid2);
	}

	return newfptr;
}

/**************************************************************/
/*                                                            */
/* Create a message data file block and read and process the  */
/* file data.                                                 */
/*                                                            */
/**************************************************************/

int getFileData(char *filename, FILEPTR* fptr)

{
	int			rc;
	int			datalen=0;
	int			remainLen;
	char		*msgdata=NULL;
	char		*msgPtr;
	char		*delimPtr;
	char		*allocPtr=0;
	FILEPTR*	newfptr;

	/* read the message data file */
	rc = readFileData(filename, &datalen, &msgdata);

	if ((0 == rc) && (msgdata != NULL) && (datalen > 0))
	{
		/* increase the file counter */
		fileCount++;

		/* point to the data we just read in */
		msgPtr = msgdata;
		remainLen = datalen;

		/* check if we have a delimiter in the data */
		delimPtr = scanForDelim(msgdata, datalen);

		if (delimPtr != NULL)
		{
			/* we must adjust the length and set the next message ptr */
			datalen = delimPtr - msgPtr;

			/* tell what we are doing */
			printf("\nfound message delimiter in file %s - length %d\n", filename, datalen);
		}

		while (remainLen > 0)
		{
			/* allocate and initialize a file pointer */
			newfptr = createFileBlock(msgPtr, allocPtr, datalen);

			/* insert it into the chain behind previous files */
			fptr->nextfile  = newfptr;
			fptr = newfptr;

			/* tell what we did */
			if (msgformat[0] > 0)
			{
				printf("mqmd format is (\"%s\")\n", msgformat);
			}

			printf("persistence = %d, code page %d, encoding %d expiry %d priority %d report %d\n", 
					persist, codepage, encoding, expiry, priority, report);

			printf("reply to QM (%s), reply to Queue (%s)\n", replyQM, replyQ);

			switch (newfptr->hasRFH)
			{
			case RFH_NO:
				{
					printf("no record format headers (rfh) to be used\n");
					break;
				}
			case RFH_V1:
				{
					if (0 == newfptr->useFileRFH)
					{
						printf("V1 record format headers (rfh) will be used\n");
						printf(" App group (%s) Format (%s)\n", rfhappgroup, rfhformat);
					}
					else
					{
						printf("V1 record format headers (rfh) found in file will be used\n");
					}

					break;
				}
			case RFH_V2:
				{
					if (0 == newfptr->useFileRFH)
					{
						printf("V2 record format headers (rfh) will be used\n");
						printf(" Domain (%s) Set (%s) Type (%s) Fmt (%s)\n", 
								rfhdomain, rfhset, rfhtype, rfhfmt);
					}
					else
					{
						printf("V2 record format headers (rfh) found in file will be used\n");
					}

					break;
				}
			case RFH_XML:
				{
					printf("XML only record format headers (rfh) will be used\n");
					break;
				}
			}

			/* see how many bytes we have left in the file */
			remainLen -= datalen + delimiterLen;

			if (remainLen > 0) 
			{
				msgPtr = delimPtr + delimiterLen;
				datalen = remainLen;
				delimPtr = scanForDelim(msgPtr, datalen);

				if (delimPtr != NULL)
				{
					/* adjust the length of the data */
					datalen = delimPtr - msgPtr;

					/* tell what we are doing */
					printf("\nfound message delimiter in file %s - length %d\n", filename, datalen);
				}

				/* see if we need to insert an RFH */
				if ((RFH_V1 == rfh) || (RFH_V2 == rfh))
				{
					/* allocate an area to hold the rfh and the data */
					allocPtr = (char *)malloc(rfhlength + datalen + 1);

					/* copy the rfh and the data to the new data area */
					memcpy(allocPtr, rfhdata, rfhlength);
					memcpy(allocPtr + rfhlength, msgPtr, datalen);

					/* set the message pointer to the new data area */
					msgPtr = allocPtr;
				}
				else
				{
					/* no need for a new data area */
					allocPtr = 0;
				}
			}
		}
	}
	else
	{
		printf("***** invalid data file name or zero length file %s *****\n", filename);
	}

	return rc;
}

void captureString(char * value, char * arg, const int maxlen)

{
	if ((int) strlen(arg) < maxlen)
	{
		strcpy(value, arg);
	}
	else
	{
		/* issue an error message */
		printf("*****Value of constant exceeds maximum length\n");

		/* Set the return code based on the message number */
		err = 94;
	}
}

int getArgPointer(int argc,
				  char *argv[],
				  int index,
				  char ** parmData,
				  const char * parmDesc)

{
	if (strlen(argv[index]) > 2)
	{
		/* point to the argument data */
		(*parmData) = argv[index] + 2;
	}
	else
	{
		index++;

		if (index < argc)
		{
			/* point to the argument data */
			(*parmData) = argv[index];
		}
		else
		{
			/* issue an error message */
			printf("*****%s parameter is missing value\n", parmDesc);
			
			/* Set the return code to indicate the error */
			err = 95;

			/*point to something valid */
			(*parmData) = argv[index - 1] + 2;
		}
	}


	return index;
}

int processNumArg(int argc,
				  char *argv[], 
				  int index, 
				  int * value, 
				  const char * parmDesc)

{
	char *argData;

	index = getArgPointer(argc, argv, index, &argData, parmDesc);

	if (index < argc)
	{
		/* convert to an integer */
		(*value) = atoi(argData);
	}

	return index;
}

int processIndArg(int argc, 
				  char *argv[], 
				  int index, 
				  char * parm, 
				  int parmsize, 
				  const char * parmDesc)

{
	char *argData;

	index = getArgPointer(argc, argv, index, &argData, parmDesc);

	if (index < argc)
	{
		/* value is included with parameter */
		captureString(parm, argData, parmsize);
	}

	return index;
}

void processArgs(int argc, char **argv)

{
	int		i;
	int		foundit;
	char	ch;

	i = 1;
	while ((i < argc) && (0 == err))
	{
		foundit = 0;

		if ('-' == argv[i][0])
		{
			ch = toupper(argv[i][1]);

			/* check for option */
			if ((0 == foundit) && ('Q' == ch))
			{
				foundit = 1;

				i = processIndArg(argc, 
								  argv, 
								  i,
								  saveQname, 
								  sizeof(saveQname), 
								  "queue name (-q)");
			}

			/* check for option */
			if ((0 == foundit) && ('M' == ch))
			{
				foundit = 1;

				i = processIndArg(argc, 
								  argv, 
								  i,
								  saveQMname, 
								  sizeof(saveQMname), 
								  "queue manager name (-m)");
			}

			/* check for option */
			if ((0 == foundit) && ('F' == ch))
			{
				foundit = 1;

				i = processIndArg(argc, 
								  argv, 
								  i,
								  parmFilename, 
								  sizeof(parmFilename), 
								  "name of parameters file (-f)");
			}

			/* check for option */
			if ((0 == foundit) && ('B' == ch))
			{
				foundit = 1;

				i = processNumArg(argc,
								  argv, 
								  i,
								  &saveBatchSize, 
								  "Messages within unit of work (-b)");

				if ((saveBatchSize < 1) || (saveBatchSize > MAX_SYNC_ALLOW))
				{
					saveBatchSize = 0;
					printf("Invalid sync point interval (%d)\n", saveBatchSize);
				}
				else
				{
					printf("Sync point interval set to %d\n", saveBatchSize);
				}
			}

			/* check for option */
			if ((0 == foundit) && ('C' == ch))
			{
				foundit = 1;

				i = processNumArg(argc,
								  argv, 
								  i,
								  &saveTotcount, 
								  "Message count (-c)");
			}
		}

		/* did we recognize the parameter? */
		if (0 == foundit)
		{
			/* issue an error message */
			printf("*****Unrecognized command line parameter found %c\n", argv[i][0]);

			/* Set the return code based on the message number */
			err = 93;
		}

		i++;
	}
}

/**************************************************************/
/*                                                            */
/* Process a parameter line that contains a parameter of type */
/* string.                                                    */
/*                                                            */
/**************************************************************/

void editCharParm(const char * parmName, 
				 char * parm, 
				 char * valueptr, 
				 int maxsize)

{
	int		len;

	len = strlen(valueptr);

	/* check if the parameter is too long */
	if (len > (maxsize - 1))
	{
		/* truncate the data */
		valueptr[maxsize - 1] = 0;

		/* issue an error message */
		printf("*****Error - parameter value length %d exceeds maximum %d for %s and was truncated\n", len, maxsize, parmName);
	}

	/* check for previous setting */
	if (parm[0] > ' ')
	{
		/* issue warning message */
		printf("Parameter %s value was previously set and has been reset to %s\n", parmName, valueptr);
	}

	/* copy the parameter value to the variable */
	memset(parm, 0, maxsize);
	strcpy(parm, valueptr);
}

int checkCharParm(const char * parmName,
				  const char * parmConst,
				  char * parm, 
				  char * valueptr, 
				  int foundit,
				  int maxsize)

{
	int		rc = 0;

	if (1 == foundit)
	{
		return 1;
	}

	if (strcmp(parmName, parmConst) == 0)
	{
		editCharParm(parmName, parm, valueptr, maxsize);
		return 1;
	}

	return 0;
}

/**************************************************************/
/*                                                            */
/* Process a parameter line that contains a parameter of type */
/* string.                                                    */
/*                                                            */
/**************************************************************/

void editYNParm(const char * parmName, 
				char * parm, 
				char * valueptr)

{
	if (('Y' == valueptr[0]) || ('y' == valueptr[0]) || ('1' == valueptr[0]))
	{
		parm[0] = 1;
	}
	else
	{
		parm[0] = 0;

		if (('N' != valueptr[0]) && ('n' != valueptr[0]) && ('0' != valueptr[0]))
		{
			/* invalid specification - issue error message */
			printf("***** Invalid specification for Yes/No parameter %s - set to No\n", parmName);
		}
	}
}

int checkYNParm(const char * parmName,
				const char * parmConst,
				char * parm, 
				char * valueptr, 
				int foundit)

{
	int		rc = 0;

	if (1 == foundit)
	{
		return 1;
	}

	if (strcmp(parmName, parmConst) == 0)
	{
		editYNParm(parmName, parm, valueptr);
		return 1;
	}

	return 0;
}

/**************************************************************/
/*                                                            */
/* Process a parameter line that contains a parameter of type */
/* hex string.                                                */
/*                                                            */
/**************************************************************/

void editHexParm(const char * parmName, 
				 char * parm, 
				 char * valueptr, 
				 int maxsize)

{
	int		len;

	len = strlen(valueptr);

	/* check if the parameter is too long */
	if (len > 2 * maxsize)
	{
		/* truncate the data */
		valueptr[2 * maxsize] = 0;

		/* issue an error message */
		printf("*****Error - parameter value length %d exceeds maximum %d for %s and was truncated\n", len, maxsize, parmName);

		len = 2 * maxsize;
	}

	/* make sure the result is an even number of bytes */
	if ((len & 1) > 0)
	{
		/* odd number of bytes, issue error and return */
		printf("***** %s truncated - odd number of bytes (%d) for hex value - %s\n", 
				parmName, len, valueptr);

		/* truncate the last character */
		len--;
	}

	memset(parm, 0, maxsize);
	HexToAscii(valueptr, len, parm);
}

int checkHexParm(const char * parmName,
				  const char * parmConst,
				  char * parm, 
				  char * valueptr, 
				  int foundit,
				  int maxsize)

{
	int		rc = 0;

	if (1 == foundit)
	{
		return 1;
	}

	if (strcmp(parmName, parmConst) == 0)
	{
		editHexParm(parmName, parm, valueptr, maxsize);
		return 1;
	}

	return 0;
}

int checkIntParm(const char * parmName,
				 const char * parmConst,
				 int * value, 
				 char * valueptr, 
				 int foundit)

{
	int		rc = 0;

	if (1 == foundit)
	{
		return 1;
	}

	if (strcmp(parmName, parmConst) == 0)
	{
		(*value) = atoi(valueptr);;
		return 1;
	}

	return 0;
}

/**************************************************************/
/*                                                            */
/* Process a line in the parameters file.                     */
/*                                                            */
/**************************************************************/

int evaluateParm(char * ptr, char * value)

{
	int		foundit=0;
	int		len;
	char	*valueptr;
	char	tempValue[32];

	/* remove any quotes from the value */
	valueptr = removeQuotes(value);
	len = strlen(valueptr);

	tempValue[0] = 0;
	if (len < sizeof(tempValue))
	{
		/* it will fit in the temporary area */
		strcpy(tempValue, valueptr);

		/* translate to upper case */
		strupper(tempValue);
	}

	foundit = checkCharParm(ptr, QNAME, qname, valueptr, foundit, sizeof(qname) - 1);
	foundit = checkCharParm(ptr, QMGR, qmname, valueptr, foundit, sizeof(qmname) - 1);
	foundit = checkCharParm(ptr, REMOTEQM, remoteQM, valueptr, foundit, sizeof(remoteQM) - 1);
	foundit = checkIntParm(ptr, MSGCOUNT, &totcount, valueptr, foundit);
	foundit = checkIntParm(ptr, TUNE, &tune, valueptr, foundit);
	foundit = checkIntParm(ptr, REPORT, &report, valueptr, foundit);
	foundit = checkCharParm(ptr, REPLYQM, replyQM, valueptr, foundit, sizeof(replyQM) - 1);
	foundit = checkCharParm(ptr, REPLYQ, replyQ, valueptr, foundit, sizeof(replyQ) - 1);
	foundit = checkCharParm(ptr, RFH_DOMAIN, rfhdomain, valueptr, foundit, sizeof(rfhdomain) - 1);
	foundit = checkCharParm(ptr, RFH_MSG_SET, rfhset, valueptr, foundit, sizeof(rfhset) - 1);
	foundit = checkCharParm(ptr, RFH_MSG_TYPE, rfhtype, valueptr, foundit, sizeof(rfhtype) - 1);
	foundit = checkCharParm(ptr, RFH_MSG_FMT, rfhfmt, valueptr, foundit, sizeof(rfhfmt) - 1);
	foundit = checkCharParm(ptr, RFH_APP_GROUP, rfhappgroup, valueptr, foundit, sizeof(rfhappgroup) - 1);
	foundit = checkCharParm(ptr, RFH_FORMAT, rfhformat, valueptr, foundit, sizeof(rfhformat) - 1);
	foundit = checkYNParm(ptr, RFH_PSC_LOCAL, &rfh_psc_local, valueptr, foundit);
	foundit = checkYNParm(ptr, RFH_PSC_NEWONLY, &rfh_psc_newonly, valueptr, foundit);
	foundit = checkYNParm(ptr, RFH_PSC_OTHERONLY, &rfh_psc_otheronly, valueptr, foundit);
	foundit = checkYNParm(ptr, RFH_PSC_ONDEMAND, &rfh_psc_ondemand, valueptr, foundit);
	foundit = checkYNParm(ptr, RFH_PSC_RETAINPUB, &rfh_psc_retainpub, valueptr, foundit);
	foundit = checkYNParm(ptr, RFH_PSC_ISRETAINPUB, &rfh_psc_isretainpub, valueptr, foundit);
	foundit = checkYNParm(ptr, RFH_PSC_CORRELID, &rfh_psc_correlid, valueptr, foundit);
	foundit = checkYNParm(ptr, RFH_PSC_DEREGALL, &rfh_psc_deregall, valueptr, foundit);
	foundit = checkYNParm(ptr, RFH_PSC_INFRETAIN, &rfh_psc_infretain, valueptr, foundit);
	foundit = checkCharParm(ptr, RFH_PSC_TOPIC1, rfh_topic1, valueptr, foundit, sizeof(rfh_topic1) - 1);
	foundit = checkCharParm(ptr, RFH_PSC_TOPIC2, rfh_topic2, valueptr, foundit, sizeof(rfh_topic2) - 1);
	foundit = checkCharParm(ptr, RFH_PSC_TOPIC3, rfh_topic3, valueptr, foundit, sizeof(rfh_topic3) - 1);
	foundit = checkCharParm(ptr, RFH_PSC_TOPIC4, rfh_topic4, valueptr, foundit, sizeof(rfh_topic4) - 1);
	foundit = checkCharParm(ptr, RFH_PSC_SUBPOINT, rfh_subpoint, valueptr, foundit, sizeof(rfh_subpoint) - 1);
	foundit = checkCharParm(ptr, RFH_PSC_FILTER, rfh_filter, valueptr, foundit, sizeof(rfh_filter) - 1);
	foundit = checkCharParm(ptr, RFH_PSC_REPLYQM, rfh_PscReplyQM, valueptr, foundit, sizeof(rfh_PscReplyQM) - 1);
	foundit = checkCharParm(ptr, RFH_PSC_REPLYQ, rfh_PscReplyQ, valueptr, foundit, sizeof(rfh_PscReplyQ) - 1);
	foundit = checkCharParm(ptr, RFH_PSC_PUBTIME, rfh_pubtime, valueptr, foundit, sizeof(rfh_pubtime) - 1);
	foundit = checkIntParm(ptr, RFH_PSC_SEQNO, &rfh_psc_seqno, valueptr, foundit);
	foundit = checkCharParm(ptr, RFH_JMS_DEST, rfh_jms_dest, valueptr, foundit, sizeof(rfh_jms_dest) - 1);
	foundit = checkCharParm(ptr, RFH_JMS_REPLY, rfh_jms_reply, valueptr, foundit, sizeof(rfh_jms_reply) - 1);
	foundit = checkCharParm(ptr, RFH_JMS_CORRELID, rfh_jms_correlid, valueptr, foundit, sizeof(rfh_jms_correlid) - 1);
	foundit = checkCharParm(ptr, RFH_JMS_GROUPID, rfh_jms_groupid, valueptr, foundit, sizeof(rfh_jms_groupid) - 1);
	foundit = checkCharParm(ptr, RFH_JMS_EXPIRE, rfh_jms_expire, valueptr, foundit, sizeof(rfh_jms_expire) - 1);
	foundit = checkCharParm(ptr, RFH_JMS_PRIORITY, rfh_jms_priority, valueptr, foundit, sizeof(rfh_jms_priority) - 1);
	foundit = checkCharParm(ptr, RFH_JMS_DELMODE, rfh_jms_delmode, valueptr, foundit, sizeof(rfh_jms_delmode) - 1);
	foundit = checkCharParm(ptr, RFH_JMS_SEQ, rfh_jms_seq, valueptr, foundit, sizeof(rfh_jms_seq) - 1);

	if (strcmp(ptr, CORRELID) == 0)
	{
		foundit = 1;
		correlidSet = 1;
		editCharParm(CORRELID, correlid, valueptr, sizeof(correlid));
	}

	if (strcmp(ptr, CORRELIDX) == 0)
	{
		foundit = 1;
		correlidSet = 1;
		editHexParm(CORRELIDX, correlid, valueptr, sizeof(correlid));
	}

	if (strcmp(ptr, GROUPID) == 0)
	{
		foundit = 1;
		groupidSet = 1;
		editCharParm(GROUPID, groupid, valueptr, sizeof(groupid));
	}

	if (strcmp(ptr, GROUPIDX) == 0)
	{
		foundit = 1;
		groupidSet = 1;
		editHexParm(GROUPIDX, groupid, valueptr, sizeof(groupid));
	}

	if (strcmp(ptr, DELIMITER) == 0)
	{
		foundit = 1;

		if (len > sizeof(delimiter))
		{
			len = sizeof(delimiter);

			/* delimiter is too long */
			printf("***** delimiter too long - value was truncated to %d\n", len);
		}

		strcpy(delimiter, valueptr);
		delimiterLen = len;
	}

	if (strcmp(ptr, DELIMITERX) == 0)
	{
		foundit = 1;

		if (len > (2 * sizeof(delimiter)))
		{
			len = 2 * sizeof(delimiter);

			/* delimiter is too long */
			printf("***** delimiter too long - value was truncated to length %d\n", len);
		}

		/* make sure the length is an even number of bytes */
		if ((len & 1) > 0)
		{
			/* odd number of bytes, issue error and return */
			printf("***** delimiterx value has odd number of bytes (%d) for hex value - %s\n", 
					len, value);
			printf("***** last character was ignored\n");

			/* truncate the last character */
			len--;
		}

		delimiterLen = len / 2;
		HexToAscii(valueptr, len, delimiter);
	}
	if (strcmp(ptr, PERSIST) == 0)
	{
		foundit = 1;
		persist = atoi(valueptr);
		if ((persist < 0) || (persist > 2))
		{
			printf("***** invalid value for persistence %d *****\n", persist);
			persist = 0;
		}
	}

	if (strcmp(ptr, ENCODING) == 0)
	{
		foundit = 1;
		encoding = atoi(valueptr);
		if (encoding < 0)
		{
			printf("***** invalid value for encoding %d *****\n", encoding);
			encoding = 546;
		}
	}

	if (strcmp(ptr, CODEPAGE) == 0)
	{
		foundit = 1;
		codepage = atoi(valueptr);
		if (codepage < 0)
		{
			printf("***** invalid value for codepage %d *****\n", codepage);
			codepage = 437;
		}
	}

	if (strcmp(ptr, EXPIRY) == 0)
	{
		foundit = 1;
		expiry = atoi(valueptr);
		if (expiry < MQEI_UNLIMITED)
		{
			printf("***** invalid value for expiration time %d *****\n", codepage);
			expiry = MQEI_UNLIMITED;
		}
	}

	if (strcmp(ptr, MSGTYPE) == 0)
	{
		foundit = 1;
		msgtype = atoi(valueptr);
		if ((msgtype != MQMT_SYSTEM_FIRST) &&
			(msgtype != MQMT_REQUEST) &&
			(msgtype != MQMT_REPLY) &&
			(msgtype != MQMT_DATAGRAM) &&
			(msgtype != MQMT_REPORT) &&
			(msgtype != MQMT_MQE_FIELDS_FROM_MQE) &&
			(msgtype != MQMT_MQE_FIELDS) &&
			((msgtype < MQMT_APPL_FIRST) || (msgtype > MQMT_APPL_LAST)))
		{
			printf("***** invalid value for message type %d *****\n", msgtype);
			msgtype = 0;
		}
	}

	if (strcmp(ptr, PRIORITY) == 0)
	{
		foundit = 1;
		priority = atoi(valueptr);
		if (priority < MQPRI_PRIORITY_AS_Q_DEF)
		{
			printf("***** invalid value for message priority %d *****\n", priority);
			priority = MQPRI_PRIORITY_AS_Q_DEF;
		}
	}

	if (strcmp(ptr, QDEPTH) == 0)
	{
		foundit = 1;
		qdepth = atoi(valueptr);
		if (qdepth < 1) 
		{
			printf("***** invalid value for queue depth %d *****\n", qdepth);
			qdepth = INIT_COUNT;
		}
	}

	if (strcmp(ptr, QMAX) == 0)
	{
		foundit = 1;
		qmax = atoi(valueptr);
		if (qmax <= qdepth) 
		{
			printf("***** invalid value for maximum queue depth %d *****\n", qmax);
			qmax = 0;
		}
	}

	if (strcmp(ptr, SLEEPTIME) == 0)
	{
		foundit = 1;
		sleeptime = atoi(valueptr);
		if ((sleeptime < MIN_SLEEP) || (sleeptime > MAX_SLEEP))
		{
			printf("***** invalid value for sleep time %d *****\n", sleeptime);
			sleeptime = MIN_SLEEP;
		}
	}

	if (strcmp(ptr, BATCHSIZE) == 0)
	{
		foundit = 1;
		batchsize = atoi(valueptr);
		if (batchsize < 0) 
		{
			printf("***** invalid value for batch size %d *****\n", batchsize);
			batchsize = 0;
		}
	}

	if (strcmp(ptr, FORMAT) == 0)
	{
		foundit = 1;
		formatSet = 1;
		editCharParm(FORMAT, msgformat, valueptr, sizeof(msgformat));
	}

	if (strcmp(ptr, RFH) == 0)
	{
		foundit = 1;
		switch (toupper(valueptr[0]))
		{
		case '1':
			{
				rfh = RFH_V1;
				break;
			}
		case '2':
			{
				rfh = RFH_V2;
				break;
			}
		case 'A':
			{
				rfh = RFH_AUTO;
				break;
			}
		case 'N':
			{
				rfh = RFH_NO;
				break;
			}
		case 'V':
			{
				if (valueptr[1] == '1')
				{
					rfh = RFH_V1;
				}
				else
				{
					if (valueptr[1] == '2')
					{
						rfh = RFH_V2;
					}
					else
					{
						printf("***** invalid value for rfh parameter %s *****\n", valueptr);
					}
				}

				break;
			}
		case 'X':
			{
				rfh = RFH_XML;
				break;
			}
		default:
			{
				printf("***** invalid value for rfh parameter %s *****\n", valueptr);
				break;
			}
		}
	}

	if (strcmp(ptr, RFH_CCSID) == 0)
	{
		foundit = 1;
		rfhccsid = atoi(valueptr);
		if (rfhccsid <= 0) 
		{
			printf("***** invalid value for rfh ccsid %d *****\n", rfhccsid);
			rfhccsid = 437;
		}
	}

	if (strcmp(ptr, RFH_ENCODING) == 0)
	{
		foundit = 1;
		rfhencoding = atoi(valueptr);
		if (rfhencoding <= 0) 
		{
			printf("***** invalid value for rfh encoding value %d *****\n", rfhencoding);
			rfhencoding = 546;
		}
	}

	if (strcmp(ptr, RFH_PSC_REQTYPE) == 0)
	{
		foundit = 1;
		if ((strcmp(tempValue, "SUBSCRIBE") == 0) || 
			(strcmp(tempValue, "SUB") == 0) ||
			(tempValue[0] == '1'))
		{
			rfh_psc_reqtype = RFH_PSC_SUB;
		}
		else
		{
			if ((strcmp(tempValue, "UNSUBSCRIBE") == 0) || 
				(strcmp(tempValue, "UNSUB") == 0) ||
				(tempValue[0] == '2'))
			{
				rfh_psc_reqtype = RFH_PSC_UNSUB;
			}
			else
			{
				if ((strcmp(tempValue, "PUBLISH") == 0) || 
					(strcmp(tempValue, "PUB") == 0) ||
					(tempValue[0] == '3'))
				{
					rfh_psc_reqtype = RFH_PSC_PUB;
				}
				else
				{
					if ((strcmp(tempValue, "REQPUB") == 0) || 
						(tempValue[0] == '4'))
					{
						rfh_psc_reqtype = RFH_PSC_REQPUB;
					}
					else
					{
						if ((strcmp(tempValue, "DELPUB") == 0) || 
							(tempValue[0] == '5'))
						{
							rfh_psc_reqtype = RFH_PSC_DELPUB;
						}
						else
						{
							printf("***** invalid value for pub/sub request type %s *****\n", valueptr);
							rfh_psc_reqtype = 0;
						}
					}
				}
			}
		}
	}

	if (strcmp(ptr, RFH_JMS_REQTYPE) == 0)
	{
		foundit = 1;
		if ((strcmp(tempValue, "TEXT") == 0) || 
			(tempValue[0] == '1'))
		{
			rfh_jms_reqtype = RFH_JMS_TEXT;
		}
		else
		{
			if ((strcmp(tempValue, "BYTES") == 0) || 
				(tempValue[0] == '2'))
			{
				rfh_jms_reqtype = RFH_JMS_BYTES;
			}
			else
			{
				if ((strcmp(tempValue, "STREAM") == 0) || 
					(tempValue[0] == '3'))
				{
					rfh_jms_reqtype = RFH_JMS_STREAM;
				}
				else
				{
					if ((strcmp(tempValue, "OBJECT") == 0) || 
						(tempValue[0] == '4'))
					{
						rfh_jms_reqtype = RFH_JMS_OBJECT;
					}
					else
					{
						if ((strcmp(tempValue, "MAP") == 0) || 
							(tempValue[0] == '5'))
						{
							rfh_jms_reqtype = RFH_JMS_MAP;
						}
						else
						{
							if ((strcmp(tempValue, "NONE") == 0) || 
								(tempValue[0] == '6'))
							{
								rfh_jms_reqtype = RFH_JMS_NONE;
							}
							else
							{
								printf("***** invalid value for jms request type %s *****\n", valueptr);
								rfh_jms_reqtype = 0;
							}
						}
					}
				}
			}
		}
	}

	return foundit;
}

/**************************************************************/
/*                                                            */
/* Find the parameter and the value.                          */
/*                                                            */
/**************************************************************/

int processParmLine(char * ptr)

{
	int		foundit=0;
	int		foundfiles=0;
	char *	valueptr;
	char *	blankptr;

	/* find the equal equal sign */
	valueptr = strchr(ptr, '=');

	/* did we find one? */
	if (valueptr != NULL)
	{
		/* break the line into name and value parts */
		valueptr[0] = 0;
		valueptr++;

		/* find the beginning of the value */
		valueptr = skipBlanks(valueptr);
	}

	/* strip trailing blanks and translate the name to upper case */
	blankptr = findBlank(ptr);
	blankptr[0] = 0;
	strupper(ptr);

	/* is this a name value pair? */
	if (valueptr != NULL)
	{
		/* check that we have a value */
		if (valueptr[0] > 0)
		{
			foundit = evaluateParm(ptr, valueptr);

			if (foundit == 0)
			{
				printf("***** unrecognized parameter %s, value %s\n", ptr, valueptr);
			}
		}
		else
		{
			printf("***** missing or blank value for parameter %s\n", ptr);
		}
	}
	else
	{
		/* check for the end of the parameters and beginning of */
		/* the list of message data files */
		if (strcmp(ptr, FILELIST) == 0)
		{
			/* remember we are in a file section */
			foundfiles = 1;
			createRFH();
		}

		/* check for message parameters section header */
		if (strcmp(ptr, HEADER) == 0)
		{
			/* remember we are in a header section */
			foundfiles = 0;
		}
	}

	return foundfiles;
}

/**************************************************************/
/*                                                            */
/* Read the parameters file and process each line.            */
/*                                                            */
/**************************************************************/

void processParmFile(char * parmFileName, FILEPTR* fptr)

{
	int		len;
	int		foundfiles=0;
	FILE *	parmfile;
	char *	ptr;
	FILEPTR* fileptr;
	char	parmline[512];

	/* initialize the parameter input area */
	memset(parmline, 0, sizeof(parmline));

	/* set the nextfile variable to the anchor */
	fileptr = fptr;

	/* open the parameters file */
	parmfile = fopen(parmFileName, "r");

	if (parmfile != NULL)
	{
		/* read the parameter file */
		while (fgets(parmline, sizeof(parmline) - 1, parmfile) != NULL)
		{
			/* check for a new line character at the end of the line */
			while ((strlen(parmline) > 0) && (parmline[strlen(parmline) - 1] < ' '))
			{
				/* get rid of the new line character */
				parmline[strlen(parmline) - 1] = 0;
			}

			/* point to the beginning of the line */
			/* skip any leading blanks */
			ptr = skipBlanks(parmline);

			/* truncate any trailing blanks */
			len = strlen(ptr);
			while ((len > 0) && (ptr[len] == ' '))
			{
				ptr[len] = 0;
				len--;
			}

			/* check for a comment or blank line */
			if ((ptr[0] != 0) && (ptr[0] != ';') && (ptr[0] != '#') && (ptr[0] != '*'))
			{
				if ((foundfiles == 0) || ('[' == ptr[0]))
				{
					/* process this line as message parameters */
					foundfiles = processParmLine(ptr);
				}
				else
				{
					/* process this line as a message data file name */
					getFileData(ptr, fileptr);

					/* check if we need to update our nextfile pointer */
					while (fileptr->nextfile != NULL)
					{
						fileptr = (FILEPTR *)fileptr->nextfile;
					}
				}
			}

			/* re-initialize the parameter input area */
			memset(parmline, 0, sizeof(parmline));
		}

		fclose(parmfile);
	}
	else
	{
		printf("***** Error opening parameters file *****\n");
		printf("***** Using default parameters *****\n");
	}
}

/**************************************************************/
/*                                                            */
/* This routine writes error messages for MQSeries errors.    */
/*                                                            */
/**************************************************************/

void checkerror(const char *mqcalltype, MQLONG compcode, MQLONG reason, char *resource)

{
	if ((compcode != MQCC_OK) || (reason != MQRC_NONE))
	{
		printf("MQSeries error with %s on %s - compcode = %d, reason = %d\n",
				mqcalltype, resource, compcode, reason);
	}
}

/**************************************************************/
/*                                                            */
/* This routine puts a message on the queue.                 */
/*                                                            */
/**************************************************************/

int putMessage(FILEPTR* fptr, 
			   MQCHAR8 *puttime, 
			   int uow)

{
	MQLONG	compcode=0;
	MQLONG	reason=0;
	MQMD	msgdesc = {MQMD_DEFAULT};
	MQPMO	mqpmo = {MQPMO_DEFAULT};

	/* set the get message options */
	if (uow > 0)
	{
		/* use syncpoints */
		mqpmo.Options = MQPMO_SYNCPOINT | MQPMO_NEW_MSG_ID | MQPMO_FAIL_IF_QUIESCING;
	}
	else
	{
		/* no synchpoint, each message as a separate UOW */
		mqpmo.Options = MQPMO_NO_SYNCPOINT | MQPMO_NEW_MSG_ID | MQPMO_FAIL_IF_QUIESCING;
	}

	/* Indicate V2 of MQMD */
	msgdesc.Version = MQMD_VERSION_2;

	/* set the persistence, etc if specified */
	msgdesc.Persistence = fptr->Persist;
	msgdesc.Encoding = fptr->Encoding;
	msgdesc.CodedCharSetId = fptr->Codepage;

	/* check if message expiry was specified */
	if (expiry > 0)
	{
		msgdesc.Expiry = fptr->Expiry;
	}

	/* check if message type was specified */
	if (msgtype > 0)
	{
		msgdesc.MsgType = fptr->Msgtype;
	}

	/* check if message priority was specified */
	if (priority != MQPRI_PRIORITY_AS_Q_DEF)
	{
		msgdesc.Priority = fptr->Priority;
	}

	/* check if report options were specified */
	if (report > 0)
	{
		msgdesc.Report = fptr->Report;
	}

	/* set the message format in the MQMD was specified */
	switch (fptr->hasRFH)
	{
	case RFH_NO:
		{
			if (1 == fptr->FormatSet)
			{
				memcpy(msgdesc.Format, fptr->Format, MQ_FORMAT_LENGTH);
			}

			break;
		}
	case RFH_V1:
		{
			memcpy(msgdesc.Format, MQFMT_RF_HEADER, sizeof(msgdesc.Format));
			break;
		}
	case RFH_V2:
		{
			memcpy(msgdesc.Format, MQFMT_RF_HEADER_2, sizeof(msgdesc.Format));
			break;
		}
	case RFH_XML:
		{
			memcpy(msgdesc.Format, MQFMT_RF_HEADER_2, sizeof(msgdesc.Format));
			break;
		}
	}

	/* check if a reply to queue manager was specified */
	memset(msgdesc.ReplyToQMgr, 0, sizeof(msgdesc.ReplyToQMgr));
	if (fptr->ReplyQM[0] != 0)
	{
		memcpy(msgdesc.ReplyToQMgr, fptr->ReplyQM, strlen(fptr->ReplyQM));
	}

	/* check if a reply to queue was specified */
	memset(msgdesc.ReplyToQ, 0, sizeof(msgdesc.ReplyToQ));
	if (fptr->ReplyQ[0] != 0)
	{
		memcpy(msgdesc.ReplyToQ, fptr->ReplyQ, strlen(fptr->ReplyQ));
	}

	/* check if a correl id was specified */
	if (correlidSet == 1)
	{
		memcpy(msgdesc.CorrelId, fptr->CorrelId, MQ_CORREL_ID_LENGTH);
	}
	else
	{
		memset(msgdesc.CorrelId, 0, MQ_CORREL_ID_LENGTH);
	}

	/* check if a group id was specified */
	if (groupidSet == 1)
	{
		memcpy(msgdesc.GroupId, fptr->GroupId, MQ_GROUP_ID_LENGTH);
		msgdesc.MsgFlags |= MQMF_LAST_MSG_IN_GROUP | MQMF_MSG_IN_GROUP ;
	}
	else
	{
		memset(msgdesc.GroupId, 0, sizeof(msgdesc.GroupId));
	}

	/* perform the MQPUT */
	MQPUT(qm, q, &msgdesc, &mqpmo, fptr->length, fptr->dataptr, &compcode, &reason);

	/* check if we got to the end of the queue */
	/* check for errors */
	checkerror("MQPUT", compcode, reason, qname);

	memcpy(puttime, msgdesc.PutTime, sizeof(msgdesc.PutTime));

	return compcode;
}

/**************************************************************/
/*                                                            */
/* This routine gets the number of messages on the queue.     */
/*                                                            */
/**************************************************************/

#ifndef NOTUNE
MQLONG getQueueDepth()

{
	MQLONG	numOnQueue=0;		/* Number of messages on Queue   */
	MQLONG	Select[1];			/* attribute selectors           */
	MQLONG	IAV[1];				/* integer attribute values      */
	MQLONG	compcode;
	MQLONG	reason;

	/* get the current queue depth */
	Select[0] = MQIA_CURRENT_Q_DEPTH;
	MQINQ(qm,			/* connection handle                 */
		  Hinq,			/* object handle                     */
		  1L,			/* Selector count                    */
		  Select,		/* Selector array                    */
		  1L,			/* integer attribute count           */
		  IAV,			/* integer attribute array           */
		  0L,			/* character attribute count         */
		  NULL,			/* character attribute array         */
		  /*  note - can use NULL because count is zero      */
		  &compcode,	/* completion code                   */
		  &reason);		/* reason code                       */

	checkerror("MQINQ", compcode, reason, qmname);
	if (compcode == MQCC_OK)
	{
		numOnQueue= IAV[0];   /* currdepth */
	}

	return numOnQueue;
}
#endif

/**************************************************************/
/*                                                            */
/* Check if the sleep time is too much or not enough.         */
/*                                                            */
/**************************************************************/

#ifndef NOTUNE
void adjustSleeptime(const int numOnQueue, const int lastdepth)

{
	int	origSleeptime = sleeptime;
	int minAdjCount;

	/* first, check if the queue is drained */
	if (numOnQueue == 0)
	{
		/* queue is empty, we need to cut the sleeptime */
		sleeptime >>= 1;
	}
	else
	{
		/* check if the counts are the same */
		if (numOnQueue == lastdepth)
		{
			/* no messages were processed */
			/* this indicates that we are checking too often */
			/* increase the sleep time by 50% */
			sleeptime += (sleeptime >> 1) + 1;
		}
		else
		{
			/* we don't want to adjust if the rate is high enough */
			/* first, calculate the difference between the desired */
			/* and maximum values */
			minAdjCount = (qmax - qdepth) >> 2;
			if ((lastdepth > numOnQueue) && ((lastdepth - numOnQueue) < minAdjCount))
			{
				/* not enough messages were processed */
				/* increase the sleeptime by a 12% */
				sleeptime += (sleeptime >> 3) + 1;
			}
		}
	}

	/* make sure we are within the allowed limits for sleeptime */
	if (sleeptime < MIN_SLEEP)
	{
		printf(" sleeptime below minimum (%d) - forced to minimum\n", sleeptime);
		sleeptime = MIN_SLEEP;
	}

	if (sleeptime > MAX_SLEEP)
	{
		printf(" sleeptime above maximum (%d) - forced to maximum\n", sleeptime);
		sleeptime = MAX_SLEEP;
	}

	/* tell what we did */
	if (origSleeptime != sleeptime)
	{
		printf(" sleeptime changed from %d to %d milliseconds numOnQueue %d lastdepth %d written %d\n", 
				origSleeptime, sleeptime, numOnQueue, lastdepth, msgwritten);
	}
}
#endif

/**************************************************************/
/*                                                            */
/* Display command format.                                    */
/*                                                            */
/**************************************************************/

void printHelp(char *pgmName)

{
	printf("%s\n", Level);
	printf("format is:\n");
	printf("  %s -f parm_file\n", pgmName);
	printf("   parm_file is the fully qualified name of the parameters file\n");
}

int main(int argc, char **argv)

{
	int		uow=0;
	int		uowcount=0;
	MQLONG	compcode;
	MQLONG	reason;
	MQOD	objdesc = {MQOD_DEFAULT};
	MQLONG	openopt = 0;
	MQLONG	datalen=0;
#ifndef NOTUNE
	int		numOnQueueMin=0;
	int		numOnQueueMax=0;
	int		lastdepth;
	int		saveCount;
	MQLONG	numOnQueue;					/* Number of messages on Queue   */
	MQLONG	O_optionsq;					/* inquire MQOPEN options        */
#endif
	MQCHAR8	puttime;
	char*	msgdata;
	char	filename[512];
	FILEPTR fptr;
	FILEPTR* fileptr;

#ifdef NOTUNE
	printf("MQPUTS V1.01 program started\n");
#else
	printf("MQPUT2 V1.01 program started\n");
#endif

	/* print the copyright statement */
	printf("Copyright (c) IBM Corp., 2001/2002\n");

	/* initialize the work areas */
	memset(parmFilename, 0, sizeof(parmFilename));
	memset(qmname, 0, sizeof(qmname));
	memset(qname, 0, sizeof(qname));
	memset(saveQMname, 0, sizeof(saveQMname));
	memset(saveQname, 0, sizeof(saveQname));
	memset(remoteQM, 0, sizeof(remoteQM));
	memset(filename, 0, sizeof(filename));
	memset(msgformat, 0, sizeof(msgformat));
	memset(rfhdata, 0, sizeof(rfhdata));
	memset(rfhdomain, 0, sizeof(rfhdomain));
	memset(rfhset, 0, sizeof(rfhset));
	memset(rfhtype, 0, sizeof(rfhtype));
	memset(rfhfmt, 0, sizeof(rfhfmt));
	memset(rfhappgroup, 0, sizeof(rfhappgroup));
	memset(rfh_PscReplyQ, 0, sizeof(rfh_PscReplyQ));
	memset(rfh_PscReplyQM, 0, sizeof(rfh_PscReplyQM));
	memset(rfh_topic1, 0, sizeof(rfh_topic1));
	memset(rfh_topic2, 0, sizeof(rfh_topic2));
	memset(rfh_topic3, 0, sizeof(rfh_topic3));
	memset(rfh_topic4, 0, sizeof(rfh_topic4));
	memset(rfh_subpoint, 0, sizeof(rfh_subpoint));
	memset(rfh_filter, 0, sizeof(rfh_filter));
	memset(rfh_pubtime, 0, sizeof(rfh_pubtime));
	memset(rfh_jms_dest, 0, sizeof(rfh_jms_dest));
	memset(rfh_jms_reply, 0, sizeof(rfh_jms_reply));
	memset(rfh_jms_correlid, 0, sizeof(rfh_jms_correlid));
	memset(rfh_jms_groupid, 0, sizeof(rfh_jms_groupid));
	memset(rfh_jms_expire, 0, sizeof(rfh_jms_expire));
	memset(rfh_jms_delmode, 0, sizeof(rfh_jms_delmode));
	memset(rfh_jms_seq, 0, sizeof(rfh_jms_seq));
	memset(rfh_jms_priority, 0, sizeof(rfh_jms_priority));

	/* initialize the message data anchor */
	fptr.dataptr = NULL;
	fptr.length = 0;
	fptr.nextfile = NULL;

	/* check for too few input parameters */
	if (argc < 2)
	{
		printHelp(argv[0]);
		exit(99);
	}

	/* check for help request */
	if ((argv[1][0] == '?') || (argv[1][1] == '?'))
	{
		printHelp(argv[0]);
		exit(0);
	}

	/* process any command line arguments */
	processArgs(argc, argv);

	if (err != 0)
	{
		printHelp(argv[0]);
		exit(99);
	}

	/* initialize the queue name and queue manager name */
	memset(qname, ' ', sizeof(qname) - 1);
	memset(qmname, ' ', sizeof(qmname) - 1);

	/* process the parameters file data */
	processParmFile(parmFilename, &fptr);

	/* check for overrides */
	if (saveQMname[0] != 0)
	{
		strcpy(qmname, saveQMname);
	}

	/* check for overrides */
	if (saveQname[0] != 0)
	{
		strcpy(qname, saveQname);
	}

	/* check for overrides */
	if (saveTotcount > 0)
	{
		totcount = saveTotcount;
	}

	/* check for overrides */
	if (saveBatchSize > 0)
	{
		batchsize = saveBatchSize;
	}

	/* tell how many files and messages we found */
	printf("Total files read %d\n", fileCount);
	printf("Total messages found %d\n", mesgCount);

	/* set the default queue depth if none was specified */
	if (qdepth == 0)
	{
		if (qmax == 0)
		{
			qdepth = INIT_COUNT;
		}
		else
		{
			qdepth = (qmax / 2) + 1;
		}
	}

	/* set the maximum depth if not specified */
	if (qmax == 0)
	{
		/* set the maximum depth to twice the minimum */
		qmax = qdepth + 16;
	}

	/* explain what parameters are being used */
	printf("\n%d messages to be written to queue %s on queue manager %s\n", 
			totcount, qname, qmname);
	printf("minimum queue depth %d max %d batchsize %d\n", qdepth, qmax, batchsize);
#ifndef NOTUNE
	printf("initial sleep time %d tune = %d\n", sleeptime, tune);
#endif
	/* check if we found any message data files */
	fileptr = (FILEPTR *)fptr.nextfile;
	if (fileptr == NULL)
	{
		printf("***** No message data files found - program terminating\n");
		return 94;
	}

	/* check if we are writing multiple messages in a single UOW */
	if (batchsize > 1)
	{
		uow = 1;
	}

	/* Connect to the queue manager */
	printf("\nconnecting to queue manager %s\n",qmname);
	MQCONN(qmname, &qm, &compcode, &reason);

	/* check for errors */
	checkerror("MQCONN", compcode, reason, qmname);
	if (compcode != MQCC_OK)
	{
		return 98;
	}

	/* set the queue open options */
	strcpy(objdesc.ObjectName, qname);
	strcpy(objdesc.ObjectQMgrName, remoteQM);
	openopt = MQOO_OUTPUT + MQOO_FAIL_IF_QUIESCING;

	/* open the queue for output */
	printf("opening queue %s for output\n", qname);
	MQOPEN(qm, &objdesc, openopt, &q, &compcode, &reason);

	/* check for errors */
	checkerror("MQOPEN", compcode, reason, qname);
	if (compcode != MQCC_OK)
	{
		return 97;
	}

	/**************************************************************/
	/*                                                            */
	/*   Open named queue for INQUIRE                             */
	/*                                                            */
	/**************************************************************/
#ifndef NOTUNE
	O_optionsq = MQOO_INQUIRE    /* open to inquire attributes     */
				 + MQOO_FAIL_IF_QUIESCING;

	MQOPEN(qm,				/* connection handle              */
		   &objdesc,		/* object descriptor for queue    */
		   O_optionsq,		/* open options                   */
		   &Hinq,			/* object handle for MQINQ        */
		   &compcode,		/* MQOPEN completion code         */
		   &reason);		/* reason code                    */

	/* check for errors */
	checkerror("MQOPEN", compcode, reason, qname);
	if (compcode != MQCC_OK)
	{
		return 96;
	}

	numOnQueue = getQueueDepth();
#endif

	/* prime the queue with an initial number of messages */
	/* After this, we will monitor the queue depth and    */
	/* try to keep the queue depth at a specified amount  */
#ifdef NOTUNE
	while ((compcode == MQCC_OK) && (msgwritten < totcount))
#else
	while ((compcode == MQCC_OK) && ((msgwritten + numOnQueue) < qmax) && (msgwritten < totcount))
#endif
	{
		/* get the data pointer and length from the next message */
		datalen = fileptr->length;
		msgdata = fileptr->dataptr;

		/* perform the MQPUT */
		compcode = putMessage(fileptr,
							  &puttime, 
							  uow);

		/* move on to the next message file */
		fileptr = (FILEPTR *)fileptr->nextfile;
		if (fileptr == NULL)
		{
			/* go back to the first message data file */
			fileptr = (FILEPTR *)fptr.nextfile;
		}

		/* check for errors */
		if (compcode == MQCC_OK)
		{
			if (msgwritten == 0)
			{
				/* write out the time of the first message */
				printf("first message written at %8.8s\n", puttime);
			}

			/* increment the message count */
			msgwritten++;

			/* check if we are using syncpoints */
			if (batchsize > 1)
			{
				/* remember how many messages are in this uow */
				uowcount++;
			}

			/* check if we nee to issue a commit */
			if ((batchsize > 1) && (uowcount == batchsize))
			{
				MQCMIT(qm, &compcode, &reason);
				checkerror("MQCMIT", compcode, reason, qname);
				uowcount = 0;
			}
		}
	}

	if (uowcount > 0)
	{
		MQCMIT(qm, &compcode, &reason);
		checkerror("MQCMIT", compcode, reason, qname);
		uowcount = 0;
	}

#ifndef NOTUNE
	/* give the initial number of messages written */
	if (tune == 1)
	{
		printf("initial number of messages written %d\n", msgwritten);
	}

	/* enter message loop */
	while ((compcode == MQCC_OK) && (msgwritten < totcount))
	{
		/* initialize the last depth variable */
		lastdepth = getQueueDepth();

		/* issue a wait for sleeptime milliseconds */
		Sleep(sleeptime); /*sleep in millisecs*/

		/* get the current queue depth */
		numOnQueue = getQueueDepth();

		/* remember the minimum and maximum counts */
		if ((numOnQueueMax == 0) || (numOnQueue < numOnQueueMin))
		{
			numOnQueueMin = numOnQueue;
		}

		if (numOnQueue > numOnQueueMax)
		{
			numOnQueueMax = numOnQueue;
		}

		/* issue error message if we find no messages on queue */
		if (numOnQueue == 0)
		{
			printf("***** warning - no messages on queue after %d msgs written\n", msgwritten);
			printf("***** decrease sleeptime parameter from %d\n", sleeptime);
		}

		/* check if we are tuning the sleeptime parameter */
		if (tune == 1)
		{
			/* give the current message count */
			printf("number on queue %d, lastdepth %d\n", numOnQueue, lastdepth);

			/* check if we want to adjust the sleep time */
			adjustSleeptime(numOnQueue, lastdepth);
		}

		/* check if we are below the minimum depth */
		if (numOnQueue < qdepth)
		{
			/* remember the number of messages written previously */
			saveCount = msgwritten;

			/* check the depth of the queue */
			while ((compcode == MQCC_OK) && (numOnQueue < qmax) && (msgwritten < totcount))
			{
				/* get the data pointer and length from the next message */
				datalen = fileptr->length;
				msgdata = fileptr->dataptr;

				/* perform the MQPUT */
				compcode = putMessage(fileptr, 
									  &puttime, 
									  uow);

				/* move on to the next message file */
				fileptr = (FILEPTR *)fileptr->nextfile;
				if (fileptr == NULL)
				{
					/* go back to the first message data file */
					fileptr = (FILEPTR *)fptr.nextfile;
				}

				/* check for errors */
				if (compcode == MQCC_OK)
				{
					/* increment the message count */
					msgwritten++;
					numOnQueue++;

					/* check if we are using syncpoints */
					if (batchsize > 1)
					{
						/* remember how many messages are in this uow */
						uowcount++;
					}

					/* check if we need to issue a commit */
					if ((batchsize > 1) && (uowcount == batchsize))
					{
						MQCMIT(qm, &compcode, &reason);
						checkerror("MQCMIT", compcode, reason, qname);
						uowcount = 0;
					}
				}
			}

			if ((batchsize > 1) && (uowcount > 0))
			{
				MQCMIT(qm, &compcode, &reason);
				checkerror("MQCMIT", compcode, reason, qname);
				uowcount = 0;
			}

			if (tune == 1)
			{
				printf("%d messages written to queue\n",msgwritten - saveCount);
			}
		}
	}

	/* check if tuning of the sleep time was requested */
	if (tune == 1)
	{
		/* write out the final sleep time value */
		printf("final sleep time value %d\n", sleeptime);
	}

	/* write out the minimum and maximum number of messages on the queue */
	printf("number on queue after sleep - min %d, max %d\n", numOnQueueMin, numOnQueueMax);
#endif

	/* write out the time of the first message */
	printf("last message written at %8.8s\n", puttime);

	/* dump out the total message count */
	printf("Total messages written %d out of %d\n", msgwritten, totcount);

	/* close the input queue */
	printf("closing the queue\n");
	MQCLOSE(qm, &q, MQCO_NONE, &compcode, &reason);

	checkerror("MQCLOSE", compcode, reason, qname);

#ifndef NOTUNE
	/* close the input queue */
	printf("closing the inquiry queue\n");
	MQCLOSE(qm, &Hinq, MQCO_NONE, &compcode, &reason);

	checkerror("MQCLOSE", compcode, reason, qname);
#endif

	/* Disconnect from the queue manager */
	printf("disconnecting from the queue manager\n");
	MQDISC(&qm, &compcode, &reason);

	checkerror("MQDISC", compcode, reason, qmname);

	/******************************************************************/
	/*                                                                */
	/* END OF PROGRAM                                                 */
	/*                                                                */
	/******************************************************************/

#ifdef NOTUNE
	printf("MQPUTS program ended\n");
#else
	printf("MQPUT2 program ended\n");
#endif

	return(0);
}
